#! /bin/sh

APPS_NAME="DSL API Application"

if [ -e sysconfig.sh ]; then
	. sysconfig.sh
	. config.sh
	. model_config.sh
	. gen_funcs.sh
	. build_functions.sh 
else
        echo "Application "$APPS_NAME" not configured"
        exit 0
fi
display_info "----------------------------------------------------------------------"
display_info "-------------------- build $APPS_NAME ----------------------------"
display_info "----------------------------------------------------------------------"

parse_args $@

cd dsl_cpe_control

if [ $BUILD_CLEAN -eq 1 ]; then
	if [ -e .config_ok ]; then
		make -s distclean
		rm -f .config_ok configure
	fi
	[ ! $BUILD_CONFIGURE -eq 1 ] && exit 0
fi
if [ ! -f .config_ok -o $BUILD_CONFIGURE -eq 1 ]; then
	if [ ! -e configure ]; then
		autoreconf -i
		if [ ! -e NEWS ]; then
		    touch NEWS
		fi
		if [ ! -e README ]; then
		    touch README
		fi
		if [ ! -e AUTHORS ]; then
		    touch AUTHORS
		fi
		if [ ! -e ChangeLog ]; then
		    touch ChangeLog
		fi
		if [ ! -e TODO ]; then
		    touch TODO
		fi
	fi
fi

if [ "$1" = "config_only" ] ;then
	exit 0
fi

KERNEL_BUILD=
DRIVER_INCLUDE=-I$PWD/../drv_dsl_cpe_api/src/include
DEVICE_DRIVER_INCLUDE=
ARCH=mips
DEVICE=danube
ADD_APPL_CFLAGS="-Os -G 0 -mlong-calls \
 -finline-limit=100000 -mabi=32 -march=mips32 -Wa,-32 -Wa,-march=mips32 \
 -Wa,-mips32 -Wa,--trap"

# call build script with defined settings
# --enable-ifxos
# --enable-ifxos-library
# --enable-ifxos-include
./configure \
    CC=$IFX_CC \
    AR=$IFX_AR \
    LD=$IFX_LD \
    AS=$IFX_AS \
    STRIP=$IFX_STRIP \
    --build=i386-linux \
    --host=$ARCH-linux-uclibc \
    --enable-add-appl-cflags="$ADD_APPL_CFLAGS" \
    --enable-driver-include="$DRIVER_INCLUDE" \
    --enable-device-driver-include="$DEVICE_DRIVER_INCLUDE" \
    --enable-$DEVICE \
    --with-channels-per-line=1 \
      --enable-debug-prints=no \
    --enable-model=full\
    --enable-debug-tool-interface=no \
    --enable-dsl-g997-per-tone=no \
    --enable-dsl-delt=no\
    --enable-soap-support=no \
    --enable-dsl-pm-channel-thresholds=no \
    --enable-dsl-pm-datapath-thresholds=no \
    --enable-dsl-pm-line-thresholds=no \
    --enable-dsl-pm-history=no \
    --enable-dsl-pm-optional-parameters=no
#    --enable-debug=yes \
#    --eble-soap-support=no \nable-debug-prints=yes \
#    --enable-debug-tool-interface \
#    --enable-soap-support=no \
#    --enable-dsl-pm \
#    --enable-dsl-pm-channel-counters \
#    --enable-dsl-pm-datapath-counters \
#    --enable-dsl-pm-line-counters \
#    --enable-dsl-pm-channel-thresholds=no \
#    --enable-dsl-pm-datapath-thresholds=no \
#    --enable-dsl-pm-line-thresholds=no \
#    --enable-dsl-pm-total \
#    --enable-dsl-pm-history=no \
#    --enable-dsl-pm-showtime \
#    --enable-dsl-pm-optional-parameters=no
#    --enable-cmv-scripts \
#    --enable-script-notification \
#    --enable-cli-support \
#    --enable-dsl-ceoc


ifx_error_check $?

make 
ifx_error_check $?

cd ../

#mkdir -p ${BUILD_ROOTFS_DIR}/usr/sbin/
#cp -af dsl_cpe_control/src/dsl_cpe_control ${BUILD_ROOTFS_DIR}/usr/sbin/

