/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN

#include "drv_dsl_cpe_api.h"

#if defined(INCLUDE_DSL_PM)

#include "drv_dsl_cpe_pm_core.h"

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_PM

DSL_boolean_t DSL_DRV_PM_IsPmReady(
   DSL_Context_t *pContext)
{
   if( DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM module not started yet!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_FALSE;
   }

   if( !DSL_DRV_PM_CONTEXT(pContext)->bInit )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM module not initialized yet!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_FALSE;
   }

   /* Check bPmLock flag*/
   if( DSL_DRV_PM_CONTEXT(pContext)->bPmLock )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d: ERROR - PM module is temporary locked!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_FALSE;
   }

   return DSL_TRUE;
}

/** \addtogroup DRV_DSL_CPE_PM
 @{ */
static DSL_Error_t DSL_DRV_PM_SyncTimeUpdate(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t msecTimeFrame = DSL_PM_COUNTER_POLLING_CYCLE,
                nCurrMsTime = 0, nCurrSysTime = 0, nPrevElapsedTime = 0;
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_SyncTimeUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Lock PM module Mutex*/
   if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex) )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock PM mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   nCurrMsTime = DSL_DRV_PM_TIME_GET();
   
   /* Check for the time change*/   
   if( DSL_DRV_PM_CONTEXT(pContext)->nLastMsTimeCheck > nCurrMsTime )
   {
      DSL_DRV_PM_CONTEXT(pContext)->nLastMsTimeCheck = nCurrMsTime;
   }

   if( DSL_DRV_PM_CONTEXT(pContext)->nLastMsTimeCheck != 0 )
   {
      /* Get elapsed time [msec] since the last entry*/
      msecTimeFrame = nCurrMsTime  - DSL_DRV_PM_CONTEXT(pContext)->nLastMsTimeCheck;
   }

   /* Get Total Elapsed Time Since the PM module startup*/
   DSL_DRV_PM_CONTEXT(pContext)->nPmTotalElapsedTime += msecTimeFrame;

   /* Set last time check to the current time*/
   DSL_DRV_PM_CONTEXT(pContext)->nLastMsTimeCheck = nCurrMsTime;

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /*
      Showtime interval processing
   */

   /* Update Line state information*/
   DSL_CTX_READ(pContext, nErrCode, nLineState, nLineState);

   if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart != DSL_TRUE )
   {
      if( nLineState == DSL_LINESTATE_SHOWTIME_NO_SYNC ||
          nLineState == DSL_LINESTATE_SHOWTIME_TC_SYNC)
      {
         /* First showtime reached*/
         DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart = DSL_TRUE;
         /* Set current showtime elapsed time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime = 0;
      }
   }
   else
   {
      /* Check for the showtime intervel trigger*/
      if( (nLineState == DSL_LINESTATE_SHOWTIME_NO_SYNC ||
           nLineState == DSL_LINESTATE_SHOWTIME_TC_SYNC) &&
           (DSL_DRV_PM_CONTEXT(pContext)->nLineState != DSL_LINESTATE_SHOWTIME_NO_SYNC &&
            DSL_DRV_PM_CONTEXT(pContext)->nLineState != DSL_LINESTATE_SHOWTIME_TC_SYNC) )
      {
         /* Set showtime Interval Trigger flag*/
         DSL_DRV_PM_CONTEXT(pContext)->bShowtimeInvTrigger = DSL_TRUE;
         /* Reset current showtime elapsed time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime = 0;
      }
      else
      {
         /* Update current showtime elapsed time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime += (msecTimeFrame/DSL_PM_MSEC);
      }
   }

   /* Update line state in the PM module context*/
   DSL_DRV_PM_CONTEXT(pContext)->nLineState = nLineState;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   switch(DSL_DRV_PM_CONTEXT(pContext)->syncMode)
   {
   case DSL_PM_SYNC_MODE_FREE:
#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
      /* add current working time in msec to avoid resync due to rounding*/
      DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime += msecTimeFrame;
      DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime  += msecTimeFrame;

      /* PM reported time will be in sec*/
      DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime/DSL_PM_MSEC;
      DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime  =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime/DSL_PM_MSEC;

      /* Check if 15 minutes interval elapsed*/
      if( DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime >=
          DSL_DRV_PM_CONTEXT(pContext)->nPm15Min)
      {
         /* Set 15 min elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b15minElapsed  = DSL_TRUE;
         /* reset current 15 min interval time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime    = 0;
      }

      /* Check if 1 day interval elapsed*/
      if( DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime >=
          DSL_DRV_PM_CONTEXT(pContext)->nPm1Day)
      {
         /* Set 1 day elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b1dayElapsed  = DSL_TRUE;
         /* reset current 1 day interval time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime = 0;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
   break;

   /*
      15-min and 1-day interval processing
   */

   case DSL_PM_SYNC_MODE_SYS_TIME:
      /* Get System Time*/
      nCurrSysTime = DSL_DRV_PM_SYS_TIME_GET();

      nPrevElapsedTime = DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime;
      /* Get curent 15-min time [sec]*/
      DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime = (nCurrSysTime % (DSL_DRV_PM_CONTEXT(pContext)->nPm15Min));

      DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime;

      /* Check if 15 min interval elapsed*/
      if( nPrevElapsedTime > DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime )
      {
         /* Set 15 min elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b15minElapsed  = DSL_TRUE;
      }

      nPrevElapsedTime = DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime;
      DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime = (nCurrSysTime % (DSL_DRV_PM_CONTEXT(pContext)->nPm1Day));

      DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime  =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime;

      /* Check if 1 day interval elapsed*/
      if( nPrevElapsedTime > DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime )
      {
         /* Set 1 day elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b1dayElapsed  = DSL_TRUE;
      }
   break;

   case DSL_PM_SYNC_MODE_EXTERNAL:
#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
      DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime += msecTimeFrame;
      DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime  += msecTimeFrame;

      /* PM reported time will be in sec*/
      DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime/DSL_PM_MSEC;
      DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime  =
         DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime/DSL_PM_MSEC;

      if( DSL_DRV_PM_CONTEXT(pContext)->bExternal15minElapsed )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: PM external 15 min sync..." DSL_DRV_CRLF  DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         /* Set 15 min elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b15minElapsed         = DSL_TRUE;
         /* Reset external 15 min elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->bExternal15minElapsed = DSL_FALSE;
         /* Reset current 15 min elapsed time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurr15MinTime = 0;
      }

      if( DSL_DRV_PM_CONTEXT(pContext)->bExternal1dayElapsed )
      {
         DSL_DEBUG(DSL_DBG_MSG,
            (pContext, "DSL[%02d]: PM external 1 day sync..." DSL_DRV_CRLF  DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         /* Set 1 day elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->b1dayElapsed  = DSL_TRUE;
         /* Reset external 1 day elapsed flag*/
         DSL_DRV_PM_CONTEXT(pContext)->bExternal1dayElapsed = DSL_FALSE;
         /* Reset current 1 day elapsed time*/
         DSL_DRV_PM_CONTEXT(pContext)->nCurr1DayTime = 0;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
   break;

   default:
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - unknown sync mode (%d)!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), DSL_DRV_PM_CONTEXT(pContext)->syncMode));
         
      nErrCode = DSL_ERROR;
   break;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

   /* Unlock PM module Mutex*/
   DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex);

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - DSL_DRV_PM_SyncTimeUpdate"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}


static DSL_void_t DSL_DRV_PM_PollCycleUpdate(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_uint32_t nStartTime,
   DSL_uint32_t nStopTime)
{
   DSL_uint32_t msecTimeFrame;
   DSL_uint32_t nPollFactor;
   DSL_PM_Thread_t *pPmThread;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_PollCycleUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if (pContext == DSL_NULL || DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL)
   {
      return;
   }

   if( nDirection == DSL_NEAR_END )
   {
      pPmThread   = &(DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe);
      nPollFactor = 1;
   }
   else
   {
      pPmThread =  &(DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe);
      nPollFactor = DSL_DRV_PM_CONTEXT(pContext)->nFeRequestCycle;
   }

   if( nStopTime < nStartTime )
   {
      /* System time has been changed*/
      pPmThread->nThreadPollTime = DSL_PM_COUNTER_POLLING_CYCLE;
   }
   else
   {
      msecTimeFrame = nStopTime - nStartTime;

      DSL_DEBUG(DSL_DBG_MSG,
         (pContext, "DSL[%02d]: PM processing time frame (%u msec)"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), msecTimeFrame));

      if( msecTimeFrame > DSL_DRV_PM_CONTEXT(pContext)->nPmTick * nPollFactor )
      {
         DSL_DEBUG(DSL_DBG_WRN,
            (pContext, "DSL[%02d]: PM %s processing out of Poll Cycle!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nDirection == DSL_NEAR_END ? "NE" : "FE"));
            
         pPmThread->nThreadPollTime = 10; /* msec*/
      }
      else
      {
         pPmThread->nThreadPollTime =
            DSL_DRV_PM_CONTEXT(pContext)->nPmTick * nPollFactor - msecTimeFrame;
      }
   }

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: PM new %s Poll Time %d msec"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection == DSL_NEAR_END ? "NE" : "FE",
      pPmThread->nThreadPollTime));
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_pm_core.h'
*/
DSL_int_t DSL_DRV_PM_ThreadNe(DSL_DRV_ThreadParams_t *param)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nOsRet = 0;
   DSL_Context_t *pContext = (DSL_Context_t*)param->nArg1;
   DSL_uint32_t startTime, stopTime;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ThreadNe" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Check DSL CPE context pointer*/
   if( pContext == DSL_NULL || DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL )
      return -1;

   /* Check if the PM was initialized*/
   if( DSL_DRV_PM_CONTEXT(pContext)->bInit != DSL_TRUE )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM module not initialized!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return -1;
   }

   DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe.bRun = DSL_TRUE;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: PM Near-End thread started"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* main PM module Task*/
   while( DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe.bRun )
   {
      /* Wake up each nThreadPollTime. nThreadPollTime is dynamically adjusted
         according to the PM processing time and PM poll cycle*/
      DSL_DRV_WAIT_EVENT_TIMEOUT( DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe.pmEvent,
                              DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe.nThreadPollTime);

      /* Get Start Time*/
      startTime = DSL_DRV_PM_TIME_GET();

      /* Lock PM module NE mutex*/
      nErrCode = DSL_DRV_PM_DirectionMutexControl(pContext, DSL_NEAR_END, DSL_TRUE);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM Near-End mutex lock failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      
         break;
      }

      /* Update PM counters for the Near-End direction*/
      nErrCode = DSL_DRV_PM_CountersUpdate(pContext, DSL_NEAR_END);

      /* Unlock PM module NE mutex*/
      DSL_DRV_PM_DirectionMutexControl(pContext, DSL_NEAR_END, DSL_FALSE);

      /* Check Update status, Reboot on Error*/
      if( nErrCode < DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM NE counters update failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         /* Restart Autoboot*/
         nErrCode =  DSL_DRV_AutobootStateSet(
            pContext, DSL_AUTOBOOTSTATE_RESTART, DSL_AUTOBOOT_RESTART_POLL_TIME);

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Autoboot state set failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
      }

      /* Update Sync Time*/
      nErrCode = DSL_DRV_PM_SyncTimeUpdate(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM Sync Time update failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

#if defined(INCLUDE_DSL_CPE_PM_HISTORY) || defined(INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS)
      /* Update history for all counters*/
      nErrCode = DSL_DRV_PM_HistoryUpdate(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM NE history update failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }
#endif /* #if defined(INCLUDE_DSL_CPE_PM_HISTORY) || defined(INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS)*/

      /* Get Stop Time*/
      stopTime = DSL_DRV_PM_TIME_GET();

      /* Update PM thread poll cycle*/
      DSL_DRV_PM_PollCycleUpdate(pContext, DSL_NEAR_END, startTime, stopTime);
   }

   /* Clear PM module bRun flag*/
   DSL_DRV_PM_CONTEXT(pContext)->pmThreadNe.bRun = DSL_FALSE;

   nOsRet = DSL_DRV_ErrorToOS(nErrCode);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ThreadNe"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nOsRet;
}

DSL_int_t DSL_DRV_PM_ThreadFe(DSL_DRV_ThreadParams_t *param)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t nOsRet = 0;
   DSL_Context_t *pContext = (DSL_Context_t*)param->nArg1;
   DSL_uint32_t startTime, stopTime;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ThreadFe" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Check DSL CPE context pointer*/
   if( pContext == DSL_NULL || DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL )
      return -1;

   /* Check if the PM was initialized*/
   if( DSL_DRV_PM_CONTEXT(pContext)->bInit != DSL_TRUE )
   {
      DSL_DEBUG(DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM module not initialized!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return -1;
   }

   DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.bRun = DSL_TRUE;

   DSL_DEBUG(DSL_DBG_MSG,
      (pContext, "DSL[%02d]: PM Far-End thread started"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* main PM module Task*/
   while( DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.bRun )
   {
      DSL_DRV_WAIT_EVENT_TIMEOUT( DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.pmEvent,
                              DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.nThreadPollTime);

      /* Get Start Time*/
      startTime = DSL_DRV_PM_TIME_GET();

      /* Lock PM module FE mutex*/
      nErrCode = DSL_DRV_PM_DirectionMutexControl(pContext, DSL_FAR_END, DSL_TRUE);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM Far-End mutex lock failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      
         break;
      }

      /* Update PM counters for the Far-End direction*/
      nErrCode = DSL_DRV_PM_CountersUpdate(pContext, DSL_FAR_END);

      /* Unlock PM module FE mutex*/
      DSL_DRV_PM_DirectionMutexControl(pContext, DSL_FAR_END, DSL_FALSE);

      /* Check Update status, Reboot on Error*/
      if( nErrCode < DSL_SUCCESS )
      {
         DSL_DEBUG(DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM FE counters update failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

         /* Restart Autoboot*/
         nErrCode =  DSL_DRV_AutobootStateSet(
            pContext, DSL_AUTOBOOTSTATE_RESTART, DSL_AUTOBOOT_RESTART_POLL_TIME);

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Autoboot state set failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
      }

      /* Get Stop Time*/
      stopTime = DSL_DRV_PM_TIME_GET();

      /* Update PM thread poll cycle*/
      DSL_DRV_PM_PollCycleUpdate(pContext, DSL_FAR_END, startTime, stopTime);
   }

   /* Clear PM module bRun flag*/
   DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.bRun = DSL_FALSE;

   nOsRet = DSL_DRV_ErrorToOS(nErrCode);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ThreadFe"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nOsRet;
}

DSL_Error_t DSL_DRV_PM_DirectionMutexControl(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_boolean_t bLock)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DirectionMutexControl,(nDirection=%d,%s)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection, bLock ? "LOCK" : "UNLOCK"));

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ATU_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module direction dependent mutex*/
   if( nDirection == DSL_NEAR_END )
   {
      if( bLock )
      {
         if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeMutex) )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock PM NE mutex!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERR_SEMAPHORE_GET;
         }
      }
      else
      {
          /* Unlock PM module NE Mutex*/
          DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeMutex);
      }
   }
   else
   {
      if( bLock )
      {
         if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeMutex) )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock PM FE mutex!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERR_SEMAPHORE_GET;
         }
      }
      else
      {
         /* Unlock PM module FE Mutex*/
         DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeMutex);
      }
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DirectionMutexControl, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_AccessMutexControl(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_boolean_t bLock)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_AccessMutexControl,(nDirection=%d,%s)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection, bLock ? "LOCK" : "UNLOCK"));

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ATU_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module direction dependent mutex*/
   if( nDirection == DSL_NEAR_END )
   {
      if( bLock )
      {
         if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeAccessMutex) )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock PM NE access mutex!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERR_SEMAPHORE_GET;
         }
      }
      else
      {
          /* Unlock PM module NE access Mutex*/
          DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeAccessMutex);
      }
   }
   else
   {
      if( bLock )
      {
         if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeAccessMutex) )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock PM FE access mutex!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            nErrCode = DSL_ERR_SEMAPHORE_GET;
         }
      }
      else
      {
         /* Unlock PM module FE access Mutex*/
         DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeAccessMutex);
      }
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_AccessMutexControl, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_Lock(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: IN - DSL_DRV_PM_Lock" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( !(DSL_DRV_PM_CONTEXT(pContext)->bPmLock) )
   {
      /* Lock PM module Near-End Mutex*/
      if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeMutex) )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock PM NE mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERR_SEMAPHORE_GET;
      }

      /* Lock PM module Near-End Mutex*/
      if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeMutex) )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Couldn't lock PM FE mutex!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return DSL_ERR_SEMAPHORE_GET;
      }

      /* Set bPmLock flag*/
      DSL_DRV_PM_CONTEXT(pContext)->bPmLock = DSL_TRUE;

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: PM locked successfully" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: PM module already locked"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_Lock, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_UnLock(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_UnLock" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( DSL_DRV_PM_CONTEXT(pContext)->bPmLock )
   {
      /* Unlock PM module NE Mutex*/
      DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmNeMutex);

      /* Unlock PM module FE Mutex*/
      DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmFeMutex);

      /* Clear bPmLock flag*/
      DSL_DRV_PM_CONTEXT(pContext)->bPmLock = DSL_FALSE;

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: PM unlocked successfully" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: PM already unlocked" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_UnLock, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#if defined(INCLUDE_DSL_CPE_PM_HISTORY) || defined(INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS)
DSL_void_t DSL_DRV_PM_HistoryCreate(
   DSL_pmHistory_t *hist,
   DSL_uint32_t historySize)
{
   DSL_ASSERT((hist && historySize));

   hist->historySize = historySize;
   hist->curItem  = 0;
   hist->itemsNum = 1;
}

DSL_Error_t DSL_DRV_PM_HistoryDelete(
   DSL_Context_t *pContext,
   DSL_pmHistory_t *hist)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, hist);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module Mutex*/
   if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex) )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Couldn't lock PM mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   hist->curItem  = 0;
   hist->itemsNum = 1;

   /* Unlock PM module Mutex*/
   DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex);

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_HistoryCurItemUpdate(
   DSL_Context_t *pContext,
   DSL_pmHistory_t *hist)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, hist);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module Mutex*/
   if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex) )
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - "
         "Couldn't lock PM mutex!"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   hist->curItem = (hist->curItem < (hist->historySize-1)) ? (hist->curItem + 1) : 0;

   if( hist->itemsNum < hist->historySize )
   {
      hist->itemsNum++;
   }

   /* Unlock PM module Mutex*/
   DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex);

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_HistoryFillLevelGet(
   DSL_Context_t *pContext,
   DSL_pmHistory_t *hist,
   DSL_uint32_t *pFillLevel)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, hist);
   DSL_CHECK_POINTER(pContext, pFillLevel);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module Mutex*/
   if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex) )
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - "
         "Couldn't lock PM mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   *pFillLevel = hist->itemsNum - 1;

   /* Unlock PM module Mutex*/
   DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex);

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_HistoryItemIdxGet(
   DSL_Context_t *pContext,
   DSL_pmHistory_t *hist,
   DSL_uint32_t histInterval,
   DSL_int_t *pIdx)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, hist);
   DSL_CHECK_POINTER(pContext, pIdx);
   DSL_CHECK_ERR_CODE();

   /* Lock PM module Mutex*/
   if( DSL_DRV_MUTEX_LOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex) )
   {
      DSL_DEBUG( DSL_DBG_ERR,(pContext, "DSL[%02d]: ERROR - "
         "Couldn't lock PM mutex!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERR_SEMAPHORE_GET;
   }

   if((histInterval > hist->historySize) || (hist->historySize == 0))
   {
      DSL_DEBUG(DSL_DBG_ERR, (DSL_NULL,
         "DSL[%02d]: PMHistory_GetItem: invalid history item number (%u of %u)!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), histInterval, hist->historySize));
         
      nErrCode = DSL_ERROR;
   }

   if (nErrCode >= DSL_SUCCESS)
   {
      *pIdx = (DSL_int_t)((hist->historySize+hist->curItem-histInterval)
         % (hist->historySize));
   }

   /* Unlock PM module Mutex*/
   DSL_DRV_MUTEX_UNLOCK(DSL_DRV_PM_CONTEXT(pContext)->pmMutex);

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_HistoryUpdate(DSL_Context_t *pContext)
{

   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_int_t histIdx = 0;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_PM_SyncEvent_t pmSyncEventData;
#endif /* INCLUDE_DSL_CPE_PM_HISTORY*/
   
   DSL_CHECK_CTX_POINTER(pContext);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_HistoryUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Lock PM module NE access mutex*/
   nErrCode = DSL_DRV_PM_AccessMutexControl(pContext, DSL_NEAR_END, DSL_TRUE);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM Near-End access mutex lock failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      
      return nErrCode;
   }

   /* Lock PM module FE access mutex*/
   nErrCode = DSL_DRV_PM_AccessMutexControl(pContext, DSL_FAR_END, DSL_TRUE);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM Far-End access mutex lock failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      
      return nErrCode;
   }

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Check if Showtime interval elapsed*/
   if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeInvTrigger )
   {
      /* Reset bShowtimeElapsed flag*/
      DSL_DRV_PM_CONTEXT(pContext)->bShowtimeInvTrigger = DSL_FALSE;
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Update Channel Counters Showtime history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_SHOWTIME() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* Get curent history index*/
      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_SHOWTIME(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime[histIdx]),
         0, sizeof(DSL_pmChannelData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      /* Update Line Counters Showtime history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_LINE_HISTORY_SHOWTIME() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_LINE_HISTORY_SHOWTIME(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime[histIdx]),
         0, sizeof(DSL_pmLineData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Update DataPath Counters Showtime history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_SHOWTIME());

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_SHOWTIME(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime[histIdx]),
         0, sizeof(DSL_pmDataPathData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   /* Clear the PM sync event data*/
   pmSyncEventData.b15MinElapsed = DSL_FALSE;
   pmSyncEventData.b1DayElapsed  = DSL_FALSE;

   /* Check if 15-min interval elapsed*/
   if( DSL_DRV_PM_CONTEXT(pContext)->b15minElapsed )
   {
      pmSyncEventData.b15MinElapsed = DSL_TRUE;
   
      /* Reset b15minElapsed flag*/
      DSL_DRV_PM_CONTEXT(pContext)->b15minElapsed = DSL_FALSE;
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Update Channel Counters 15-min history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_15MIN() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* Get curent history index*/
      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_15MIN(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min[histIdx]),
         0, sizeof(DSL_pmChannelData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      /* Update Line Counters 15-min history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_LINE_HISTORY_15MIN() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_LINE_HISTORY_15MIN(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min[histIdx]),
         0, sizeof(DSL_pmLineData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Update DataPath Counters 15-min history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_15MIN());

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_15MIN(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min[histIdx]),
         0, sizeof(DSL_pmDataPathData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   }

   /* Check if 1-day interval elapsed*/
   if( DSL_DRV_PM_CONTEXT(pContext)->b1dayElapsed )
   {
      pmSyncEventData.b1DayElapsed  = DSL_TRUE;
      
      /* Reset b1dayElapsed flag*/
      DSL_DRV_PM_CONTEXT(pContext)->b1dayElapsed = DSL_FALSE;
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Update Channel Counters 1-day history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext,  DSL_DRV_PM_PTR_CHANNEL_HISTORY_1DAY() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* Get curent history index*/
      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_1DAY(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - channel history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day[histIdx]),
         0, sizeof(DSL_pmChannelData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      /* Update Line Counters 1-day history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_LINE_HISTORY_1DAY() );

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_LINE_HISTORY_1DAY(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - line history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      /* set history memory to zeros */
      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day[histIdx]),
         0, sizeof(DSL_pmLineData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Update DataPath Counters 1-day history*/
      nErrCode = DSL_DRV_PM_HistoryCurItemUpdate( pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_1DAY());

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item update failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                    pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_1DAY(), 0, &histIdx);

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - data path history item index get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }

      memset(
         &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day[histIdx]),
         0, sizeof(DSL_pmDataPathData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   }

   if ((pmSyncEventData.b15MinElapsed) || (pmSyncEventData.b1DayElapsed))
   {
      /* Signal 15-min elapsed condition by the DSL_EVENT_S_PM_SYNC event*/
      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, DSL_XTUDIR_NA,
                    DSL_EVENT_S_PM_SYNC,
                    (DSL_EventData_Union_t*)&pmSyncEventData,
                    sizeof(DSL_PM_SyncEvent_t));

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - DSL_EVENT_S_PM_SYNC event generate failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         goto on_hist_update_error;
      }
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

on_hist_update_error:
   /* Unlock PM module NE access mutex*/
   DSL_DRV_PM_AccessMutexControl(pContext, DSL_NEAR_END, DSL_FALSE);
   /* Unlock PM module FE access mutex*/
   DSL_DRV_PM_AccessMutexControl(pContext, DSL_FAR_END, DSL_FALSE);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_HistoryUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
DSL_Error_t DSL_DRV_PM_HistoryStatsGet(
   DSL_Context_t *pContext,
   DSL_pmHistory_t *pHist,
   DSL_PM_HistoryStatsData_t *pStats)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t histFillLevel = 0;
   DSL_CHECK_POINTER(pContext, pHist);
   DSL_CHECK_POINTER(pContext, pStats);
   DSL_CHECK_ERR_CODE();

   nErrCode = DSL_DRV_PM_HistoryFillLevelGet( pContext, pHist, &histFillLevel);

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - history fill level get failed!!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   /* Get statistics on 1-day history*/
   pStats->nPrevIvs        = histFillLevel;
   pStats->nPrevInvalidIvs = pHist->historySize - pStats->nPrevIvs - 1;

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
#endif /* #if defined(INCLUDE_DSL_CPE_PM_HISTORY) || defined(INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS)*/

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_THRESHOLDS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
/* Function to check the Channel Counters thresholds crossing for 15-min and 1-day
    interval counters. Appropriate event will be generated on the thresholds crossing
    condition*/
static DSL_Error_t DSL_DRV_PM_ChannelCountersThresholdsCrossingCheck(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *p15mCounters,
   DSL_PM_ChannelData_t *p1dCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_ChannelThresholdCrossing_t channelThresholdCrossing;
   DSL_uint8_t i = 0;
   DSL_uint32_t *pEvtThresholdInd, *pThresholdInd;
   DSL_pmChannelThresholdCrossingData_t *pInd ;
   DSL_PM_ChannelData_t *pThresholds;
   DSL_PM_ChannelData_t *pCounters = DSL_NULL;
   DSL_boolean_t bEvent = DSL_FALSE;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, p15mCounters);
   DSL_CHECK_POINTER(pContext, p1dCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ChannelCountersThresholdsCrossingCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   pInd = DSL_DRV_PM_PTR_CHANNEL_THRESHOLD_IND(nDirection);
   pInd = &(pInd[nChannel]);

   channelThresholdCrossing.n15Min = DSL_PM_CHANNELTHRESHCROSS_EMPTY;
   channelThresholdCrossing.n1Day  = DSL_PM_CHANNELTHRESHCROSS_EMPTY;

   for (i = 0; i < 2; i++)
   {
      if (i == 0)
      {
         pEvtThresholdInd = &(channelThresholdCrossing.n15Min);
         pThresholdInd    = &(pInd->n15Min);
         pThresholds      = DSL_DRV_PM_PTR_CHANNEL_THRESHOLD_15MIN(nDirection);
         pCounters        = p15mCounters;
      }
      else
      {
         pEvtThresholdInd = &(channelThresholdCrossing.n1Day);
         pThresholdInd    = &(pInd->n1Day);
         pThresholds      = DSL_DRV_PM_PTR_CHANNEL_THRESHOLD_1DAY(nDirection);
         pCounters        = p1dCounters;
      }

      pThresholds = &(pThresholds[nChannel]);

      if( pCounters->nFEC > pThresholds->nFEC)
         *pEvtThresholdInd |= DSL_PM_CHANNELTHRESHCROSS_FEC;

      if( pCounters->nCodeViolations > pThresholds->nCodeViolations )
         *pEvtThresholdInd |= DSL_PM_CHANNELTHRESHCROSS_CV;

      /* Check if new threshold crossing occured since the last check*/
      if( *pEvtThresholdInd != *pThresholdInd )
      {
         *pThresholdInd = *pEvtThresholdInd;
         bEvent = DSL_TRUE;
      }
   }

   /* Check if at least one threshold crossing occured*/
   if (bEvent)
   {
      /* Get current PM module time*/
      channelThresholdCrossing.nCurr15MinTime = DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime;
      channelThresholdCrossing.nCurr1DayTime  = DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime;

      /* Generate DSL_EVENT_I_CHANNEL_THRESHOLD_CROSSING event*/
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, nChannel, DSL_ACCESSDIR_NA, nDirection, DSL_EVENT_I_CHANNEL_THRESHOLD_CROSSING,
                     (DSL_EventData_Union_t*)&channelThresholdCrossing,
                     sizeof(DSL_PM_ChannelThresholdCrossing_t));
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_ChannelCountersThresholdsCrossingCheck, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   Function to update Channel Counters thresholds in the PM context
*/
DSL_Error_t DSL_DRV_PM_ChannelThresholdsUpdate(
   DSL_PM_ChannelData_t *pCounters,
   DSL_PM_ChannelData_t *pThreshsOld,
   DSL_PM_ChannelData_t *pThreshsNew,
   DSL_uint32_t *pInd)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if( pCounters && pThreshsOld && pThreshsNew && pInd )
   {
      /* Set new nFEC threshold*/
      pThreshsOld->nFEC = pThreshsNew->nFEC;
      if( pCounters->nFEC <= pThreshsOld->nFEC )
         /* Clear nFEC indication bitmask*/
         *pInd &= ~DSL_PM_CHANNELTHRESHCROSS_FEC;

      /* Set new nCodeViolations threshold*/
      pThreshsOld->nCodeViolations = pThreshsNew->nCodeViolations;
      if( pCounters->nCodeViolations <= pThreshsOld->nCodeViolations )
         /* Clear nCodeViolations indication bitmask*/
         *pInd &= ~DSL_PM_CHANNELTHRESHCROSS_CV;
   }
   else
   {
      nErrCode = DSL_ERROR;
   }

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_THRESHOLDS*/

static DSL_Error_t DSL_DRV_PM_ChannelCountersCurrentGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ChannelCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Call device specific stuff*/
   nErrCode = DSL_DRV_PM_DEV_ChannelCountersGet(
                 pContext, nChannel, nDirection, pCounters);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ChannelCountersUpdate, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

static DSL_Error_t DSL_DRV_PM_ChannelCountersUpdate(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pNewCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif
   DSL_PM_ChannelData_t *pCurrCounters, *pAbsCounters, *pTotalCounters;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_PM_ChannelData_t *p15minCounters, *p1dayCounters;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_PM_ChannelData_t *pShowtimeCounters;
#endif

   DSL_CHECK_POINTER(pContext, pNewCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ChannelCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_1DAY(),
                 0, &hist1dayIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   if( hist15minIdx < 0  || hist1dayIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Channel history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY */

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   if( histShowtimeIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Channel Showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   /* Get counters from the PM module context*/
   pAbsCounters      = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_ABS(nChannel,nDirection);
   pCurrCounters     = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_CURR(nChannel,nDirection);
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   p15minCounters    = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_15MIN(hist15minIdx,nChannel,nDirection);
   p1dayCounters     = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_1DAY(hist1dayIdx,nChannel,nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
   pTotalCounters    = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_TOTAL(nChannel,nDirection);
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   pShowtimeCounters = DSL_DRV_PM_PTR_CHANNEL_COUNTERS_SHOWTIME(histShowtimeIdx,nChannel,nDirection);

   /* Update Showtime Timeframe history information*/
   DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.nShowtimeTimeHist[histShowtimeIdx] =
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime;
#endif

   /* Update counters only if the PM data is valid*/
   if (DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid)
   {
      /* Check if the TR1 condition is fulfilled for the FE direction*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bFeTr1Reset && nDirection == DSL_FAR_END )
      {
         /* Update Absolute values*/
         pAbsCounters->nFEC            = pNewCounters->nFEC;
         pAbsCounters->nCodeViolations = pNewCounters->nCodeViolations;
      }
      else
      {
         /* Check for the Wrap Around condition*/
         if (pNewCounters->nFEC)
         {
            /* Update Absolute value*/
            pAbsCounters->nFEC =
               (pNewCounters->nFEC < pCurrCounters->nFEC) ?
               (pAbsCounters->nFEC + pNewCounters->nFEC +
               (DSL_PM_COUNTER_FEC_MAX_VALUE - pCurrCounters->nFEC)) :
               (pNewCounters->nFEC - pCurrCounters->nFEC);
            pCurrCounters->nFEC = pNewCounters->nFEC;
         }

         /* Check for the Wrap Around condition*/
         if (pNewCounters->nCodeViolations)
         {
            /* Update Absolute value*/
           pAbsCounters->nCodeViolations =
               (pNewCounters->nCodeViolations < pCurrCounters->nCodeViolations) ?
               (pAbsCounters->nCodeViolations + pNewCounters->nCodeViolations +
               (DSL_PM_COUNTER_CODEVIOLATIONS_MAX_VALUE - pCurrCounters->nCodeViolations)) :
               (pNewCounters->nCodeViolations - pCurrCounters->nCodeViolations);
            pCurrCounters->nCodeViolations = pNewCounters->nCodeViolations;
         }
      }

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Update 15-min counters*/
      p15minCounters->nFEC            += pAbsCounters->nFEC;
      p15minCounters->nCodeViolations += pAbsCounters->nCodeViolations;

      /* Update 1-day counters*/
      p1dayCounters->nFEC             += pAbsCounters->nFEC;
      p1dayCounters->nCodeViolations  += pAbsCounters->nCodeViolations;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

      /* Update Total counters*/
      pTotalCounters->nFEC            += pAbsCounters->nFEC;
      pTotalCounters->nCodeViolations += pAbsCounters->nCodeViolations;

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Update Showtime counters*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart )
      {
         pShowtimeCounters->nFEC            += pAbsCounters->nFEC;
         pShowtimeCounters->nCodeViolations += pAbsCounters->nCodeViolations;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   }
   
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_THRESHOLDS
   #ifdef INCLUDE_DSL_CPE_PM_HISTORY
   #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
   /* Check for the Channel Counters 15-min and 1-day interval Thresholds Crossing*/
   nErrCode = DSL_DRV_PM_ChannelCountersThresholdsCrossingCheck(
                 pContext, nChannel, nDirection, p15minCounters, p1dayCounters);
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_THRESHOLDS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ChannelCountersUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   Function to reset Channel Counters
*/
DSL_Error_t DSL_DRV_PM_ChannelCountersReset(
   DSL_Context_t *pContext,
   DSL_PM_ResetTypes_t ResetType)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ChannelCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_1DAY(),
                 0, &hist1dayIdx);

   if( hist15minIdx < 0  || hist1dayIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Channel history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);

   if( histShowtimeIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Channel showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/


   switch(ResetType)
   {
   case DSL_PM_RESET_CURRENT:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   /* Reset current 15-min and 1-day values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min[hist15minIdx]),
          0x0, sizeof(DSL_pmChannelData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day[hist1dayIdx]),
          0x0, sizeof(DSL_pmChannelData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Reset current showtime values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime[histShowtimeIdx]),
          0x0, sizeof(DSL_pmChannelData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
      break;

   case DSL_PM_RESET_HISTORY:
   case DSL_PM_RESET_ALL:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Delete all history intervals*/
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_15MIN() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM channel history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_1DAY() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM channel history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         /* Reset 15-min and 1-day values curent and history*/
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min), 
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_15MIN_RECORDS_NUM + 1));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day), 
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_1DAY_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min[hist15minIdx]),
            sizeof(DSL_pmChannelData_t));

         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day[hist1dayIdx]),
            sizeof(DSL_pmChannelData_t));

         /* Reset 15-min and 1-day history values*/
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec15min[1]),
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_15MIN_RECORDS_NUM));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.rec1day[1]), 
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_1DAY_RECORDS_NUM));
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_CHANNEL_HISTORY_SHOWTIME() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM channel history showtime delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime), 
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_SHOWTIME_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime[histShowtimeIdx]),
            sizeof(DSL_pmChannelData_t));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recShowtime[1]), 
                0x0, 
                sizeof(DSL_pmChannelData_t) * (DSL_PM_CHANNEL_SHOWTIME_RECORDS_NUM));
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   if( ResetType == DSL_PM_RESET_HISTORY )
      break;

   case DSL_PM_RESET_TOTAL:
#ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS
      memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recTotal), 
             0x0, 
             sizeof(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recTotal));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS*/
   break;

   default:
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown reset type!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
      
      break;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ChannelCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/


static DSL_Error_t DSL_DRV_PM_LineCountersCurrentGet(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_pmLineData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_LineSecData_t  *pLineSecCounters = DSL_NULL;
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
   DSL_PM_LineInitData_t *pLineInitCounters = DSL_NULL;
#endif

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_LineCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
   /* Proceed Init counters*/
   if( nDirection == DSL_NEAR_END )
   {
      /* Get pointer to the Line Init counters*/
      pLineInitCounters  = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS(pCounters);

      /* Get current Line Init Counters*/
      nErrCode = DSL_DRV_PM_DEV_LineInitCountersGet(pContext, pLineInitCounters);
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
   /* Proceed Sec counters*/
   if( nErrCode == DSL_SUCCESS )
   {
      /* Get pointer to the Line Sec counters according to the specified direction*/
      pLineSecCounters = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS(pCounters, nDirection);

      /* Get current Line Sec Counters*/
      nErrCode = DSL_DRV_PM_DEV_LineSecCountersGet(pContext, nDirection, pLineSecCounters);
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_LineCountersUpdate, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_LINE_THRESHOLDS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
/* Function to check the Line Counters thresholds crossing for 15-min and 1-day
    interval counters. Appropriate event will be generated on the thresholds crossing
    condition*/
static DSL_Error_t DSL_DRV_PM_LineCountersThresholdsCrossingCheck(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_PM_LineInitData_t *p15mLineInitCounters,
   DSL_PM_LineSecData_t *p15mLineSecCounters,
   DSL_PM_LineInitData_t *p1dLineInitCounters,
   DSL_PM_LineSecData_t *p1dLineSecCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_LineThresholdCrossing_t lineThresholdCrossing;
   DSL_uint32_t *pEvtThresholdInd, *pThresholdInd;
   DSL_pmLineThresholdCrossingData_t *pInd ;
   DSL_PM_LineSecData_t *pLineSecThresholds;
   DSL_PM_LineInitData_t *pLineInitThresholds;
   DSL_PM_LineInitData_t *pLineInitCounters;
   DSL_PM_LineSecData_t *pLineSecCounters;
   DSL_boolean_t bEvent = DSL_FALSE;
   DSL_uint8_t i = 0;
#ifdef INCLUDE_DSL_ADSL_MIB
   DSL_MIB_ADSL_Thresholds_t nMibThresholds = 0, nCurrMibThresholds;
#endif

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, p15mLineSecCounters);
   DSL_CHECK_POINTER(pContext, p1dLineSecCounters);

   if( nDirection == DSL_NEAR_END )
   {
      DSL_CHECK_POINTER(pContext, p15mLineInitCounters);
      DSL_CHECK_POINTER(pContext, p1dLineInitCounters);
   }
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_LineCountersThresholdsCrossingCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   pInd = DSL_DRV_PM_PTR_LINE_THRESHOLD_IND(nDirection);

   lineThresholdCrossing.n15Min = DSL_PM_LINETHRESHCROSS_EMPTY;
   lineThresholdCrossing.n1Day  = DSL_PM_LINETHRESHCROSS_EMPTY;

   for (i = 0; i< 2; i++)
   {
      if( i != 0 )
      {
         pEvtThresholdInd    = &(lineThresholdCrossing.n15Min);
         pThresholdInd       = &(pInd->n15Min);
         pLineInitThresholds = DSL_DRV_PM_PTR_LINE_INIT_THRESHOLD_15MIN();
         pLineSecThresholds  = DSL_DRV_PM_PTR_LINE_SEC_THRESHOLD_15MIN(nDirection);
		 pLineInitCounters   = p15mLineInitCounters;
		 pLineSecCounters    = p15mLineSecCounters;
      }
      else
      {
         pEvtThresholdInd    = &(lineThresholdCrossing.n1Day);
         pThresholdInd       = &(pInd->n1Day);
         pLineInitThresholds = DSL_DRV_PM_PTR_LINE_INIT_THRESHOLD_1DAY();
         pLineSecThresholds  = DSL_DRV_PM_PTR_LINE_SEC_THRESHOLD_1DAY(nDirection);
		 pLineInitCounters   = p1dLineInitCounters;
		 pLineSecCounters    = p1dLineSecCounters;
      }

      if( nDirection == DSL_NEAR_END )
      {
         if( pLineInitCounters->nFailedFullInits > pLineInitThresholds->nFailedFullInits )
            *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_FIFAIL;
   
         if( pLineInitCounters->nFailedShortInits >
            pLineInitThresholds->nFailedShortInits )
         {
            *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_SIFAIL;
         }

         if( pLineInitCounters->nFullInits > pLineInitThresholds->nFullInits )
            *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_FI;

         if( pLineInitCounters->nShortInits > pLineInitThresholds->nShortInits )
         {
            #ifdef INCLUDE_DSL_ADSL_MIB
            nMibThresholds |= DSL_MIB_THRESHOLD_ATUC_15MIN_FAILED_FASTR_FLAG;
            #endif
            *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_SI;
         }
      }

      if( pLineSecCounters->nES > pLineSecThresholds->nES )
      {
         *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_ES;
         #ifdef INCLUDE_DSL_ADSL_MIB
         nMibThresholds |= (nDirection == DSL_NEAR_END) ?
            DSL_MIB_THRESHOLD_ATUR_ESS_FLAG : DSL_MIB_THRESHOLD_ATUC_ESS_FLAG;
         #endif
      }

      if( pLineSecCounters->nLOFS > pLineSecThresholds->nLOFS )
      {
         *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_LOFS;
         #ifdef INCLUDE_DSL_ADSL_MIB
         nMibThresholds |= (nDirection == DSL_NEAR_END) ?
            DSL_MIB_THRESHOLD_ATUR_LOFS_FLAG : DSL_MIB_THRESHOLD_ATUC_LOFS_FLAG;
         #endif
      }

      if( pLineSecCounters->nLOSS > pLineSecThresholds->nLOSS )
      {
         *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_LOSS;
         #ifdef INCLUDE_DSL_ADSL_MIB
         nMibThresholds |= (nDirection == DSL_NEAR_END) ?
            DSL_MIB_THRESHOLD_ATUR_LOSS_FLAG : DSL_MIB_THRESHOLD_ATUC_LOSS_FLAG;
         #endif
      }

      if( pLineSecCounters->nSES > pLineSecThresholds->nSES )
      {
         *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_SES;
         #ifdef INCLUDE_DSL_ADSL_MIB
         nMibThresholds |= (nDirection == DSL_NEAR_END) ?
            DSL_MIB_THRESHOLD_ATUR_15MIN_SESL_FLAG : 
            DSL_MIB_THRESHOLD_ATUC_15MIN_SESL_FLAG;
         #endif
      }

      if( pLineSecCounters->nUAS > pLineSecThresholds->nUAS )
      {
         *pEvtThresholdInd |= DSL_PM_LINETHRESHCROSS_UAS;
         #ifdef INCLUDE_DSL_ADSL_MIB
         nMibThresholds |= (nDirection == DSL_NEAR_END) ?
            DSL_MIB_THRESHOLD_ATUR_15MIN_UASL_FLAG : 
            DSL_MIB_THRESHOLD_ATUC_15MIN_UASL_FLAG;
         #endif
      }

      /* Check if new threshold crossing occured since the last check*/
      if( *pEvtThresholdInd != *pThresholdInd )
      {
         *pThresholdInd = *pEvtThresholdInd;
		 bEvent = DSL_TRUE;
      }
   }

   #ifdef INCLUDE_DSL_ADSL_MIB
   DSL_CTX_READ_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
      nCurrMibThresholds);
   DSL_CTX_WRITE_SCALAR(pContext, nErrCode, MibAdslCtx.nThresholds,
      nMibThresholds | nCurrMibThresholds);
   #endif

   /* Check if at least one threshold crossing occured*/
   if (bEvent)
   {
      /* Get current PM module time*/
      lineThresholdCrossing.nCurr15MinTime = DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime;
      lineThresholdCrossing.nCurr1DayTime  = DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime;

      /* Generate DSL_EVENT_I_LINE_THRESHOLD_CROSSING event*/
      nErrCode = DSL_DRV_EventGenerate(
                    pContext, 0, DSL_ACCESSDIR_NA, nDirection, DSL_EVENT_I_LINE_THRESHOLD_CROSSING,
                    (DSL_EventData_Union_t*)&lineThresholdCrossing,
                    sizeof(DSL_PM_LineThresholdCrossing_t));
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_LineCountersThresholdsCrossingCheck, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   Function to update Line Sec Counters thresholds in the PM context
*/
DSL_Error_t DSL_DRV_PM_LineSecThresholdsUpdate(
   DSL_PM_LineSecData_t *pCounters,
   DSL_PM_LineSecData_t *pThreshsOld,
   DSL_PM_LineSecData_t *pThreshsNew,
   DSL_uint32_t *pInd)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if( pCounters && pThreshsOld && pThreshsNew && pInd )
   {
      /* Set new nES threshold*/
      pThreshsOld->nES = pThreshsNew->nES;
      if( pCounters->nES <= pThreshsOld->nES )
         /* Clear nES indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_ES;

      /* Set new nLOFS threshold*/
      pThreshsOld->nLOFS = pThreshsNew->nLOFS;
      if( pCounters->nLOFS <= pThreshsOld->nLOFS )
         /* Clear nLOFS indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_LOFS;

      /* Set new nLOSS threshold*/
      pThreshsOld->nLOSS = pThreshsNew->nLOSS;
      if( pCounters->nLOSS <= pThreshsOld->nLOSS )
         /* Clear nLOSS indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_LOSS;

      /* Set new nSES threshold*/
      pThreshsOld->nSES = pThreshsNew->nSES;
      if( pCounters->nSES <= pThreshsOld->nSES )
         /* Clear nSES indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_SES;

      /* Set new nUAS threshold*/
      pThreshsOld->nUAS = pThreshsNew->nUAS;
      if( pCounters->nUAS <= pThreshsOld->nUAS )
         /* Clear nUAS indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_UAS;
   }
   else
   {
      nErrCode = DSL_ERROR;
   }

   return nErrCode;
}

/*
   Function to update Line Init Counters thresholds in the PM context
*/
DSL_Error_t DSL_DRV_PM_LineInitThresholdsUpdate(
   DSL_PM_LineInitData_t *pCounters,
   DSL_PM_LineInitData_t *pThreshsOld,
   DSL_PM_LineInitData_t *pThreshsNew,
   DSL_uint32_t *pInd)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if( pCounters && pThreshsOld && pThreshsNew && pInd )
   {
      /* Set new nFailedFullInits threshold*/
      pThreshsOld->nFailedFullInits = pThreshsNew->nFailedFullInits;
      if( pCounters->nFailedFullInits <= pThreshsOld->nFailedFullInits )
         /* Clear nFailedFullInits indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_FIFAIL;

      /* Set new nFailedShortInits threshold*/
      pThreshsOld->nFailedShortInits = pThreshsNew->nFailedShortInits;
      if( pCounters->nFailedShortInits <= pThreshsOld->nFailedShortInits )
         /* Clear nFailedShortInits indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_SIFAIL;

      /* Set new nFullInits threshold*/
      pThreshsOld->nFullInits = pThreshsNew->nFullInits;
      if( pCounters->nFullInits <= pThreshsOld->nFullInits )
         /* Clear nFullInits indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_FI;

      /* Set new nShortInits threshold*/
      pThreshsOld->nShortInits = pThreshsNew->nShortInits;
      if( pCounters->nShortInits <= pThreshsOld->nShortInits )
         /* Clear nShortInits indication bitmask*/
         *pInd &= ~DSL_PM_LINETHRESHCROSS_SI;
   }
   else
   {
      nErrCode = DSL_ERROR;
   }

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_THRESHOLDS*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

static DSL_Error_t DSL_DRV_PM_LineCountersUpdate(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection,
   DSL_pmLineData_t *pNewCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   DSL_PM_LineInitData_t *pCurrInitCounters = DSL_NULL, *pAbsInitCounters = DSL_NULL,
                         *pTotalInitCounters = DSL_NULL;

   #ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_PM_LineInitData_t *p15minInitCounters = DSL_NULL, *p1dayInitCounters = DSL_NULL;
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
   #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_PM_LineInitData_t *pShowtimeInitCounters = DSL_NULL;
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_PM_LineSecData_t  *p15minSecCounters = DSL_NULL, *p1daySecCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_PM_LineSecData_t  *pShowtimeSecCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   DSL_PM_LineSecData_t  *pTotalSecCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
   DSL_PM_LineSecData_t  *pCurrSecCounters = DSL_NULL, *pNewSecCounters = DSL_NULL,
                         *pAbsSecCounters = DSL_NULL;

   DSL_pmLineSecAuxData_t *pLineSecAuxData;
   DSL_G997_BF_LineFailures_t nLineFailures = 0;
   DSL_uint32_t nCurrTime = 0;
#ifndef DSL_CPE_PM_SECOND_IMPLEMENTATION
   DSL_LineStateValue_t nCurrentState = DSL_LINESTATE_UNKNOWN;
#endif

   DSL_CHECK_POINTER(pContext, pNewCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_ChannelCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( nDirection == DSL_NEAR_END )
   {
      /* Get Line Failures information for the Near-End*/
      DSL_CTX_READ(pContext, nErrCode, nLineFailuresNe, nLineFailures);
   }
   else
   {
      /* Get Line Failures information for the Far-End*/
      DSL_CTX_READ(pContext, nErrCode, nLineFailuresFe, nLineFailures);
   }

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_1DAY(),
                 0, &hist1dayIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   if( hist15minIdx < 0  || hist1dayIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Line history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   if( histShowtimeIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Line Showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Update Showtime Timeframe history information*/
   DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.nShowtimeTimeHist[histShowtimeIdx] =
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   if( nDirection == DSL_NEAR_END )
   {
      /* Get counters from the PM module context*/
      pAbsInitCounters      = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_ABS();
      pCurrInitCounters     = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_CURR();
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      p15minInitCounters    = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_15MIN(hist15minIdx);
      p1dayInitCounters     = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_1DAY(hist1dayIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      pShowtimeInitCounters = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_SHOWTIME(histShowtimeIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
      pTotalInitCounters    = DSL_DRV_PM_PTR_LINE_INIT_COUNTERS_TOTAL();

      /* Check for the Wrap Around condition*/
      if (pNewCounters->init.nFailedFullInits)
      {
         /* Update Absolute value*/
         pAbsInitCounters->nFailedFullInits =
            (pNewCounters->init.nFailedFullInits < pCurrInitCounters->nFailedFullInits) ?
            (pAbsInitCounters->nFailedFullInits + pNewCounters->init.nFailedFullInits +
            (DSL_PM_COUNTER_FFINIT_MAX_VALUE - pCurrInitCounters->nFailedFullInits)) :
            (pNewCounters->init.nFailedFullInits - pCurrInitCounters->nFailedFullInits);
        pCurrInitCounters->nFailedFullInits = pNewCounters->init.nFailedFullInits;
      }

      /* ...for the nFailedShortInits counter*/
      if (pNewCounters->init.nFailedShortInits)
      {
         /* Update Absolute value*/
         pAbsInitCounters->nFailedShortInits =
            (pNewCounters->init.nFailedShortInits < pCurrInitCounters->nFailedShortInits) ?
            (pAbsInitCounters->nFailedShortInits + pNewCounters->init.nFailedShortInits +
            (DSL_PM_COUNTER_FSINIT_MAX_VALUE - pCurrInitCounters->nFailedShortInits)) :
            (pNewCounters->init.nFailedShortInits - pCurrInitCounters->nFailedShortInits);
         pCurrInitCounters->nFailedShortInits = pNewCounters->init.nFailedShortInits;
      }

      /* ...for the nFullInits counter*/
      if (pNewCounters->init.nFullInits)
      {
         /* Update Absolute value*/
         pAbsInitCounters->nFullInits =
            (pNewCounters->init.nFullInits < pCurrInitCounters->nFullInits) ?
            (pAbsInitCounters->nFullInits + pNewCounters->init.nFullInits +
            (DSL_PM_COUNTER_FINIT_MAX_VALUE - pCurrInitCounters->nFullInits)) :
            (pNewCounters->init.nFullInits - pCurrInitCounters->nFullInits);
         pCurrInitCounters->nFullInits = pNewCounters->init.nFullInits;
      }

      /* ...for the nShortInits counter*/
      if (pNewCounters->init.nShortInits)
      {
         /* Update Absolute value*/
         pAbsInitCounters->nShortInits =
            (pNewCounters->init.nShortInits < pCurrInitCounters->nShortInits) ?
            (pAbsInitCounters->nShortInits + pNewCounters->init.nShortInits +
            (DSL_PM_COUNTER_SINIT_MAX_VALUE - pCurrInitCounters->nShortInits)) :
            (pNewCounters->init.nShortInits - pCurrInitCounters->nShortInits);
         pCurrInitCounters->nShortInits = pNewCounters->init.nShortInits;
      }

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Update 15-min counters*/
      p15minInitCounters->nFailedFullInits   += pAbsInitCounters->nFailedFullInits;
      p15minInitCounters->nFailedShortInits  += pAbsInitCounters->nFailedShortInits;
      p15minInitCounters->nFullInits         += pAbsInitCounters->nFullInits;
      p15minInitCounters->nShortInits        += pAbsInitCounters->nShortInits;

      /* Update 1-day counters*/
      p1dayInitCounters->nFailedFullInits    += pAbsInitCounters->nFailedFullInits;
      p1dayInitCounters->nFailedShortInits   += pAbsInitCounters->nFailedShortInits;
      p1dayInitCounters->nFullInits          += pAbsInitCounters->nFullInits;
      p1dayInitCounters->nShortInits         += pAbsInitCounters->nShortInits;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Update Showtime counters*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart )
      {
         pShowtimeInitCounters->nFailedFullInits  += pAbsInitCounters->nFailedFullInits;
         pShowtimeInitCounters->nFailedShortInits += pAbsInitCounters->nFailedShortInits;
         pShowtimeInitCounters->nFullInits        += pAbsInitCounters->nFullInits;
         pShowtimeInitCounters->nShortInits       += pAbsInitCounters->nShortInits;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

      /* Update Total counters*/
      pTotalInitCounters->nFailedFullInits   += pAbsInitCounters->nFailedFullInits;
      pTotalInitCounters->nFailedShortInits  += pAbsInitCounters->nFailedShortInits;
      pTotalInitCounters->nFullInits         += pAbsInitCounters->nFullInits;
      pTotalInitCounters->nShortInits        += pAbsInitCounters->nShortInits;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

   /* Get counters from the PM module context*/
   pCurrSecCounters     = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_CURR(nDirection);
   pAbsSecCounters      = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_ABS(nDirection);
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   p15minSecCounters    = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_15MIN(hist15minIdx,nDirection);
   p1daySecCounters     = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_1DAY(hist1dayIdx,nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   pShowtimeSecCounters = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_SHOWTIME(histShowtimeIdx,nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   pTotalSecCounters    = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_TOTAL(nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
   pLineSecAuxData      = DSL_DRV_PM_PTR_LINE_SEC_AUX_DATA(nDirection);

   pNewSecCounters = nDirection ==DSL_NEAR_END ? &(pNewCounters->sec_ne) :
                                                 &(pNewCounters->sec_fe);

   /*
      Special handling for the LOF counter
   */
   if( (nLineFailures & DSL_G997_LINEFAILURE_LOF) &&
       (!pLineSecAuxData->nLOFBegin) )
   {
      pLineSecAuxData->nLOFBegin = DSL_DRV_PM_TIME_GET()/DSL_PM_MSEC;
   }
   else if( !(nLineFailures & DSL_G997_LINEFAILURE_LOF) )
   {
      /* Update counters and clear timestamp */
      if( pLineSecAuxData->nLOFBegin )
      {
         nCurrTime = (DSL_DRV_PM_TIME_GET()/DSL_PM_MSEC);
         pLineSecAuxData->nLOFIntern += pLineSecAuxData->nLOFBegin > nCurrTime ? 
                                        nCurrTime :
                                        nCurrTime - pLineSecAuxData->nLOFBegin;
         pLineSecAuxData->nLOFCurr    = 0;
         pLineSecAuxData->nLOFBegin   = 0;
      }
   }
   else
   {
      nCurrTime = (DSL_DRV_PM_TIME_GET()/DSL_PM_MSEC);
      pLineSecAuxData->nLOFCurr = pLineSecAuxData->nLOFBegin > nCurrTime ?
                                  nCurrTime :
                                  nCurrTime - pLineSecAuxData->nLOFBegin;
   }

   pNewSecCounters->nLOFS += (pLineSecAuxData->nLOFIntern + pLineSecAuxData->nLOFCurr);

#ifndef DSL_CPE_PM_SECOND_IMPLEMENTATION
   /*
      Special handling for the UAS counter
   */
   /* Get current line state*/
   DSL_CTX_READ_SCALAR(pContext, nErrCode, nLineState, nCurrentState);

   if( ((nCurrentState != DSL_LINESTATE_SHOWTIME_TC_SYNC)  &&
        (nCurrentState != DSL_LINESTATE_SHOWTIME_NO_SYNC)) &&
        (!pLineSecAuxData->nUASBegin))
   {
      pLineSecAuxData->nUASBegin = DSL_DRV_PM_TIME_GET();
   }
   else if( ((nCurrentState == DSL_LINESTATE_SHOWTIME_TC_SYNC)  ||
             (nCurrentState == DSL_LINESTATE_SHOWTIME_NO_SYNC)) )
   {
      if( pLineSecAuxData->nUASBegin )
      {
         nCurrTime = DSL_DRV_PM_TIME_GET();
         pLineSecAuxData->nUASIntern += (pLineSecAuxData->nUASBegin > nCurrTime ?
                                         nCurrTime :
                                         nCurrTime - pLineSecAuxData->nUASBegin)/DSL_PM_MSEC;
         pLineSecAuxData->nUASBegin   = 0;
         pLineSecAuxData->nUASCurr    = 0;
      }
   }
   else
   {
      nCurrTime = DSL_DRV_PM_TIME_GET(); 
      pLineSecAuxData->nUASCurr = (pLineSecAuxData->nUASBegin > nCurrTime ?
                                   nCurrTime :
                                   nCurrTime - pLineSecAuxData->nUASBegin)/DSL_PM_MSEC;
   }

   pNewSecCounters->nUAS = (pLineSecAuxData->nUASIntern + pLineSecAuxData->nUASCurr);
#endif /* #ifndef DSL_CPE_PM_SECOND_IMPLEMENTATION*/

   /* Update counters only if the PM data is valid*/
   if (DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid)
   {
      /* Check if the TR1 condition is fulfilled for the FE direction*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bFeTr1Reset && nDirection == DSL_FAR_END )
      {
         /* Update Absolute values*/
         pAbsSecCounters->nES   = pNewSecCounters->nES;
         pAbsSecCounters->nLOFS = pNewSecCounters->nLOFS;
         pAbsSecCounters->nLOSS = pNewSecCounters->nLOSS;
         pAbsSecCounters->nSES  = pNewSecCounters->nSES;
         pAbsSecCounters->nUAS  = pNewSecCounters->nUAS;
      }
      else
      {
         /* Check for the Wrap Around condition*/
         if (pNewSecCounters->nES)
         {
            /* Update Absolute value*/
            pAbsSecCounters->nES =
               (pNewSecCounters->nES < pCurrSecCounters->nES) ?
               (pAbsSecCounters->nES + pNewSecCounters->nES +
               (DSL_PM_COUNTER_ES_MAX_VALUE - pCurrSecCounters->nES)) :
               (pNewSecCounters->nES - pCurrSecCounters->nES);
            pCurrSecCounters->nES = pNewSecCounters->nES;
         }

         /* ...for the nLOFS counter*/
         if (pNewSecCounters->nLOFS)
         {
            pAbsSecCounters->nLOFS =
               (pNewSecCounters->nLOFS < pCurrSecCounters->nLOFS) ?
               (pAbsSecCounters->nLOFS + pNewSecCounters->nLOFS +
               (DSL_PM_COUNTER_LOFS_MAX_VALUE - pCurrSecCounters->nLOFS)) :
               (pNewSecCounters->nLOFS - pCurrSecCounters->nLOFS);
            pCurrSecCounters->nLOFS = pNewSecCounters->nLOFS;
         }

         /* ...for the nLOSS counter*/
         if (pNewSecCounters->nLOSS)
         {
            pAbsSecCounters->nLOSS =
               (pNewSecCounters->nLOSS < pCurrSecCounters->nLOSS) ?
               (pAbsSecCounters->nLOSS + pNewSecCounters->nLOSS +
               (DSL_PM_COUNTER_LOSS_MAX_VALUE - pCurrSecCounters->nLOSS)) :
               (pNewSecCounters->nLOSS - pCurrSecCounters->nLOSS);
            pCurrSecCounters->nLOSS = pNewSecCounters->nLOSS;
         }

         /* ...for the nSES counter*/
         if (pNewSecCounters->nSES)
         {
            pAbsSecCounters->nSES =
               (pNewSecCounters->nSES < pCurrSecCounters->nSES) ?
               (pAbsSecCounters->nSES + pNewSecCounters->nSES +
               (DSL_PM_COUNTER_SES_MAX_VALUE - pCurrSecCounters->nSES)) :
               (pNewSecCounters->nSES - pCurrSecCounters->nSES);
            pCurrSecCounters->nSES = pNewSecCounters->nSES;
         }

         /* ...for the nUAS counter*/
         if (pNewSecCounters->nUAS)
         {
            pAbsSecCounters->nUAS =
               (pNewSecCounters->nUAS < pCurrSecCounters->nUAS) ?
               (pAbsSecCounters->nUAS + pNewSecCounters->nUAS +
               (DSL_PM_COUNTER_UAS_MAX_VALUE - pCurrSecCounters->nUAS)) :
               (pNewSecCounters->nUAS - pCurrSecCounters->nUAS);
            pCurrSecCounters->nUAS = pNewSecCounters->nUAS;
         }
      }

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Update 15-min counters*/
      p15minSecCounters->nES   += pAbsSecCounters->nES;
      p15minSecCounters->nLOFS += pAbsSecCounters->nLOFS;
      p15minSecCounters->nLOSS += pAbsSecCounters->nLOSS;
      p15minSecCounters->nSES  += pAbsSecCounters->nSES;
      p15minSecCounters->nUAS  += pAbsSecCounters->nUAS;

      /* Update 1-day counters*/
      p1daySecCounters->nES   += pAbsSecCounters->nES;
      p1daySecCounters->nLOFS += pAbsSecCounters->nLOFS;
      p1daySecCounters->nLOSS += pAbsSecCounters->nLOSS;
      p1daySecCounters->nSES  += pAbsSecCounters->nSES;
      p1daySecCounters->nUAS  += pAbsSecCounters->nUAS;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Update Showtime counters*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart )
      {
         pShowtimeSecCounters->nES   += pAbsSecCounters->nES;
         pShowtimeSecCounters->nLOFS += pAbsSecCounters->nLOFS;
         pShowtimeSecCounters->nLOSS += pAbsSecCounters->nLOSS;
         pShowtimeSecCounters->nSES  += pAbsSecCounters->nSES;
         pShowtimeSecCounters->nUAS  += pAbsSecCounters->nUAS;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
      /* Update Total counters*/
      pTotalSecCounters->nES   += pAbsSecCounters->nES;
      pTotalSecCounters->nLOFS += pAbsSecCounters->nLOFS;
      pTotalSecCounters->nLOSS += pAbsSecCounters->nLOSS;
      pTotalSecCounters->nSES  += pAbsSecCounters->nSES;
      pTotalSecCounters->nUAS  += pAbsSecCounters->nUAS;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
  }
  
#ifdef INCLUDE_DSL_CPE_PM_LINE_THRESHOLDS
   #ifdef INCLUDE_DSL_CPE_PM_HISTORY
   #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS

   /* Check for the Channel Counters Thresholds Crossing*/
   nErrCode = DSL_DRV_PM_LineCountersThresholdsCrossingCheck(
                 pContext, nDirection,
                 p15minInitCounters,
                 p15minSecCounters,
                 p1dayInitCounters,
                 p1daySecCounters);
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_THRESHOLDS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_ChannelCountersUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
/*
   Function to reset Line Counters
*/
DSL_Error_t DSL_DRV_PM_LineCountersReset(
   DSL_Context_t *pContext,
   DSL_PM_ResetTypes_t ResetType)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_LineCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_1DAY(),
                 0, &hist1dayIdx);

   if( hist15minIdx < 0  || hist1dayIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Channel history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_LINE_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);

   if( histShowtimeIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Channel showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/


   switch(ResetType)
   {
   case DSL_PM_RESET_CURRENT:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   /* Reset current 15-min and 1-day values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min[hist15minIdx]),
          0x0, sizeof(DSL_pmLineData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day[hist1dayIdx]),
          0x0, sizeof(DSL_pmLineData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Reset current showtime values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime[histShowtimeIdx]),
          0x0, sizeof(DSL_pmLineData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
      break;

   case DSL_PM_RESET_HISTORY:
   case DSL_PM_RESET_ALL:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Delete all history intervals*/
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_LINE_HISTORY_15MIN() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM line history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_LINE_HISTORY_1DAY() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM line history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_15MIN_RECORDS_NUM + 1));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_1DAY_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min[hist15minIdx]),
            sizeof(DSL_pmLineData_t));

         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day[hist1dayIdx]),
            sizeof(DSL_pmLineData_t));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec15min[1]),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_15MIN_RECORDS_NUM));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.rec1day[1]),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_1DAY_RECORDS_NUM));
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Delete all history intervals*/
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_LINE_HISTORY_SHOWTIME() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM line history showtime delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_SHOWTIME_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime[histShowtimeIdx]),
            sizeof(DSL_pmLineData_t));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recShowtime[1]),
                0x0,
                sizeof(DSL_pmLineData_t) * (DSL_PM_LINE_SHOWTIME_RECORDS_NUM));
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   if( ResetType == DSL_PM_RESET_HISTORY )
      break;

   case DSL_PM_RESET_TOTAL:
#ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS
      memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recTotal), 
             0x0, 
             sizeof(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recTotal));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS*/
      break;

   default:
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown reset type!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
      
      break;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_LineCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_THRESHOLDS
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
#ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
/* Function to check the Data Path Counters thresholds crossing for 15-min or 1-day
    interval counters. Appropriate event will be generated on the thresholds crossing
    condition*/
static DSL_Error_t DSL_DRV_PM_DataPathCountersThresholdsCrossingCheck(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *p15mCounters,
   DSL_PM_DataPathData_t *p1dCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_DataPathThresholdCrossing_t dataPathThresholdCrossing;
   DSL_uint32_t *pEvtThresholdInd, *pThresholdInd;
   DSL_pmDataPathThresholdCrossingData_t *pInd ;
   DSL_PM_DataPathData_t *pThresholds;
   DSL_PM_DataPathData_t *pCounters = DSL_NULL;
   DSL_boolean_t bEvent = DSL_FALSE;
   DSL_uint8_t i = 0;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_POINTER(pContext, p15mCounters);
   DSL_CHECK_POINTER(pContext, p1dCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DataPathCountersThresholdsCrossingCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   pInd = DSL_DRV_PM_PTR_DATAPATH_THRESHOLD_IND(nDirection);
   pInd = &(pInd[nChannel]);

   dataPathThresholdCrossing.n15Min = DSL_PM_DATAPATHTHRESHCROSS_EMPTY;
   dataPathThresholdCrossing.n1Day  = DSL_PM_DATAPATHTHRESHCROSS_EMPTY;

   for (i = 0; i < 2; i++)
   {
      if (i == 0)
      {
         pEvtThresholdInd = &(dataPathThresholdCrossing.n15Min);
         pThresholdInd    = &(pInd->n15Min);
         pThresholds      = DSL_DRV_PM_PTR_DATAPATH_THRESHOLD_15MIN(nDirection);
         pCounters        = p15mCounters;
      }
      else
      {
         pEvtThresholdInd = &(dataPathThresholdCrossing.n1Day);
         pThresholdInd    = &(pInd->n1Day);
         pThresholds      = DSL_DRV_PM_PTR_DATAPATH_THRESHOLD_1DAY(nDirection);
         pCounters        = p1dCounters;
      }

      pThresholds = &(pThresholds[nChannel]);

      /* Check for nCRC_P threshold crossing*/
      if( pCounters->nCRC_P > pThresholds->nCRC_P )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_CRC_P;

      /* Check for nCRCP_P threshold crossing*/
      if( pCounters->nCRCP_P > pThresholds->nCRCP_P)
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_CRCP_P;

      /* Check for nCV_P threshold crossing*/
      if( pCounters->nCV_P > pThresholds->nCV_P )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_CV_P;

      /* Check for nCVP_P threshold crossing*/
      if( pCounters->nCVP_P > pThresholds->nCVP_P )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_CVP_P;

      /* Check for nHEC threshold crossing*/
      if( pCounters->nHEC > pThresholds->nHEC )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_HEC;

      /* Check for nIBE threshold crossing*/
      if( pCounters->nIBE > pThresholds->nIBE )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_IBE;

      /* Check for nTotalCells threshold crossing*/
      if( pCounters->nTotalCells > pThresholds->nTotalCells )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_TOTALCELL;

      /* Check for nUserTotalCells threshold crossing*/
      if( pCounters->nUserTotalCells > pThresholds->nUserTotalCells )
         *pEvtThresholdInd |= DSL_PM_DATAPATHTHRESHCROSS_USERCELL;

      /* Check if new threshold crossing occured since the last check*/
      if( *pEvtThresholdInd != *pThresholdInd )
      {
         *pThresholdInd = *pEvtThresholdInd;
         bEvent = DSL_TRUE;
      }
   }

   /* Check if at least one threshold crossing occured*/
   if (bEvent)
   {
      /* Get current PM module time*/
      dataPathThresholdCrossing.nCurr15MinTime = DSL_DRV_PM_CONTEXT(pContext)->nElapsed15MinTime;
      dataPathThresholdCrossing.nCurr1DayTime  = DSL_DRV_PM_CONTEXT(pContext)->nElapsed1DayTime;

      /* Generate DSL_EVENT_I_DATA_PATH_THRESHOLD_CROSSING event*/
      nErrCode = DSL_DRV_EventGenerate(
                     pContext, nChannel, DSL_ACCESSDIR_NA, nDirection,
                     DSL_EVENT_I_DATA_PATH_THRESHOLD_CROSSING,
                     (DSL_EventData_Union_t*)&dataPathThresholdCrossing,
                     sizeof(DSL_PM_DataPathThresholdCrossing_t));
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_DataPathCountersThresholdsCrossingCheck, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   Function to update Channel Counters thresholds in the PM context
*/
DSL_Error_t DSL_DRV_PM_DataPathThresholdsUpdate(
   DSL_PM_DataPathData_t *pCounters,
   DSL_PM_DataPathData_t *pThreshsOld,
   DSL_PM_DataPathData_t *pThreshsNew,
   DSL_uint32_t *pInd)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   if( pCounters && pThreshsOld && pThreshsNew && pInd )
   {
      /* Set new nCRC_P threshold*/
      pThreshsOld->nCRC_P = pThreshsNew->nCRC_P;
      if( pCounters->nCRC_P <= pThreshsOld->nCRC_P )
         /* Clear nCRC_P indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_CRC_P;

      /* Set new nCRCP_P threshold*/
      pThreshsOld->nCRCP_P = pThreshsNew->nCRCP_P;
      if( pCounters->nCRCP_P <= pThreshsOld->nCRCP_P )
         /* Clear nCRCP_P indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_CRCP_P;

      /* Set new nCV_P threshold*/
      pThreshsOld->nCV_P = pThreshsNew->nCV_P;
      if( pCounters->nCV_P <= pThreshsOld->nCV_P )
         /* Clear nCV_P indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_CV_P;

      /* Set new nCVP_P threshold*/
      pThreshsOld->nCVP_P = pThreshsNew->nCVP_P;
      if( pCounters->nCVP_P <= pThreshsOld->nCVP_P )
         /* Clear nCVP_P indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_CVP_P;

      /* Set new nHEC threshold*/
      pThreshsOld->nHEC = pThreshsNew->nHEC;
      if( pCounters->nHEC <= pThreshsOld->nHEC )
         /* Clear nHEC indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_HEC;

      /* Set new nIBE threshold*/
      pThreshsOld->nIBE = pThreshsNew->nIBE;
      if( pCounters->nIBE <= pThreshsOld->nIBE )
         /* Clear nIBE indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_IBE;

      /* Set new nTotalCells threshold*/
      pThreshsOld->nTotalCells = pThreshsNew->nTotalCells;
      if( pCounters->nTotalCells <= pThreshsOld->nTotalCells )
         /* Clear nTotalCells indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_TOTALCELL;

      /* Set new nTotalCells threshold*/
      pThreshsOld->nUserTotalCells = pThreshsNew->nUserTotalCells;
      if( pCounters->nUserTotalCells <= pThreshsOld->nUserTotalCells )
         /* Clear nUserTotalCells indication bitmask*/
         *pInd &= ~DSL_PM_DATAPATHTHRESHCROSS_USERCELL;
   }
   else
   {
      nErrCode = DSL_ERROR;
   }

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_THRESHOLDS*/

static DSL_Error_t DSL_DRV_PM_DataPathCountersCurrentGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_CHANNEL_RANGE(nChannel);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DataPathCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   nErrCode = DSL_DRV_PM_DEV_DataPathCountersGet(
                 pContext, nChannel, nDirection, pCounters);

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_DataPathCountersUpdate, nChannel=%d retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nChannel, nErrCode));

   return nErrCode;
}

static DSL_Error_t DSL_DRV_PM_DataPathCountersUpdate(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   const DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pNewCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_PM_DataPathData_t *pCurrCounters, *pTotalCounters,
                         *pAbsCounters;

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_PM_DataPathData_t *p15minCounters, *p1dayCounters;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_PM_DataPathData_t *pShowtimeCounters;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_CHECK_POINTER(pContext, pNewCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DataPathCountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_1DAY(),
                 0, &hist1dayIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   if( hist15minIdx < 0  || hist1dayIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Data Path history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   if( histShowtimeIdx < 0 )
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: ERROR - Data Path Showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   /* Get counters from the PM module context*/
   pAbsCounters      = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_ABS(nChannel,nDirection);
   pCurrCounters     = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_CURR(nChannel,nDirection);
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   p15minCounters    = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_15MIN(hist15minIdx,nChannel,nDirection);
   p1dayCounters     = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_1DAY(hist1dayIdx,nChannel,nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   pShowtimeCounters = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_SHOWTIME(histShowtimeIdx,nChannel,nDirection);
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   pTotalCounters    = DSL_DRV_PM_PTR_DATAPATH_COUNTERS_TOTAL(nChannel,nDirection);

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Update Showtime Timeframe history information*/
   DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.nShowtimeTimeHist[histShowtimeIdx] =
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtimeTime;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   /* Update counters only if the PM data is valid*/
   if (DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid)
   {
      /* Check if the TR1 condition is fulfilled for the FE direction*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bFeTr1Reset && nDirection == DSL_FAR_END )
      {
         /* Update Absolute values*/
         pAbsCounters->nCRC_P          = pNewCounters->nCRC_P;
         pAbsCounters->nCRCP_P         = pNewCounters->nCRCP_P;
         pAbsCounters->nCV_P           = pNewCounters->nCV_P;
         pAbsCounters->nCVP_P          = pNewCounters->nCVP_P;
         pAbsCounters->nHEC            = pNewCounters->nHEC;
         pAbsCounters->nIBE            = pNewCounters->nIBE;
         pAbsCounters->nTotalCells     = pNewCounters->nTotalCells;
         pAbsCounters->nUserTotalCells = pNewCounters->nUserTotalCells;
      }
      else
      {
         /* Check for the Wrap Around condition for the nCRC_P counter*/
         if (pNewCounters->nCRC_P)
         {
            /* Update Absolute value*/
            pAbsCounters->nCRC_P = 
               (pNewCounters->nCRC_P < pCurrCounters->nCRC_P) ? 
               (pAbsCounters->nCRC_P + pNewCounters->nCRC_P +
               (DSL_PM_COUNTER_CRCP_MAX_VALUE - pCurrCounters->nCRC_P)) :
               (pNewCounters->nCRC_P - pCurrCounters->nCRC_P);
            pCurrCounters->nCRC_P = pNewCounters->nCRC_P;
         }

         /* ...for the nCRCP_P counter*/
         if (pNewCounters->nCRCP_P)
         {
            /* Update Absolute value*/
            pAbsCounters->nCRCP_P =
               (pNewCounters->nCRCP_P < pCurrCounters->nCRCP_P) ?
               (pAbsCounters->nCRCP_P + pNewCounters->nCRCP_P +
               (DSL_PM_COUNTER_CRCPP_MAX_VALUE - pCurrCounters->nCRCP_P)) :
               (pNewCounters->nCRCP_P - pCurrCounters->nCRCP_P);
            pCurrCounters->nCRCP_P = pNewCounters->nCRCP_P;
         }

         /* ...for the nCV_P counter*/
         if (pNewCounters->nCV_P)
         {
            /* Update Absolute value*/
            pAbsCounters->nCV_P =
               (pNewCounters->nCV_P < pCurrCounters->nCV_P) ?
               (pAbsCounters->nCV_P + pNewCounters->nCV_P +
               (DSL_PM_COUNTER_CVP_MAX_VALUE - pCurrCounters->nCV_P)) :
                                  (pNewCounters->nCV_P - pCurrCounters->nCV_P);
            pCurrCounters->nCV_P = pNewCounters->nCV_P;
         }

         /* ...for the nCVP_P counter*/
         if (pNewCounters->nCVP_P)
         {
            /* Update Absolute value*/
            pAbsCounters->nCVP_P =
               (pNewCounters->nCVP_P < pCurrCounters->nCVP_P) ?
               (pAbsCounters->nCVP_P + pNewCounters->nCVP_P +
               (DSL_PM_COUNTER_CVPP_MAX_VALUE - pCurrCounters->nCVP_P)) :
               (pNewCounters->nCVP_P - pCurrCounters->nCVP_P);
            pCurrCounters->nCVP_P = pNewCounters->nCVP_P;
         }

         /* ...for the nHEC counter*/
         if (pNewCounters->nHEC)
         {
            /* Update Absolute value*/
            pAbsCounters->nHEC =
               (pNewCounters->nHEC < pCurrCounters->nHEC) ?
               (pAbsCounters->nHEC + pNewCounters->nHEC +
               (DSL_PM_COUNTER_HEC_MAX_VALUE - pCurrCounters->nHEC)) :
               (pNewCounters->nHEC - pCurrCounters->nHEC);
            pCurrCounters->nHEC = pNewCounters->nHEC;
         }

         /* ...for the nIBE counter*/
         if (pNewCounters->nIBE)
         {
            /* Update Absolute value*/
            pAbsCounters->nIBE =
               (pNewCounters->nIBE < pCurrCounters->nIBE) ?
               (pAbsCounters->nIBE + pNewCounters->nIBE +
               (DSL_PM_COUNTER_IBE_MAX_VALUE - pCurrCounters->nIBE)) :
               (pNewCounters->nIBE - pCurrCounters->nIBE);
            pCurrCounters->nIBE = pNewCounters->nIBE;
         }

         /* ...for the nTotalCells counter*/
         if (pNewCounters->nTotalCells)
         {
            /* Update Absolute value*/
            pAbsCounters->nTotalCells =
               (pNewCounters->nTotalCells < pCurrCounters->nTotalCells) ?
               (pAbsCounters->nTotalCells + pNewCounters->nTotalCells +
               (DSL_PM_COUNTER_TOTALCELL_MAX_VALUE - pCurrCounters->nTotalCells)) :
               (pNewCounters->nTotalCells - pCurrCounters->nTotalCells);
            pCurrCounters->nTotalCells = pNewCounters->nTotalCells;
         }

         /* ...for the nUserTotalCells counter*/
         if (pNewCounters->nUserTotalCells)
         {
            /* Update Absolute value*/
            pAbsCounters->nUserTotalCells =
               (pNewCounters->nUserTotalCells < pCurrCounters->nUserTotalCells) ?
               (pAbsCounters->nUserTotalCells + pNewCounters->nUserTotalCells +
               (DSL_PM_COUNTER_UTOTALCELL_MAX_VALUE - pCurrCounters->nUserTotalCells)) :
               (pNewCounters->nUserTotalCells - pCurrCounters->nUserTotalCells);
            pCurrCounters->nUserTotalCells = pNewCounters->nUserTotalCells;
         }
      }

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Update 15-min counters*/
      p15minCounters->nCRC_P          += pAbsCounters->nCRC_P;
      p15minCounters->nCRCP_P         += pAbsCounters->nCRCP_P;
      p15minCounters->nCV_P           += pAbsCounters->nCV_P;
      p15minCounters->nCVP_P          += pAbsCounters->nCVP_P;
      p15minCounters->nHEC            += pAbsCounters->nHEC;
      p15minCounters->nIBE            += pAbsCounters->nIBE;
      p15minCounters->nTotalCells     += pAbsCounters->nTotalCells;
      p15minCounters->nUserTotalCells += pAbsCounters->nUserTotalCells;

      /* Update 1-day counters*/
      p1dayCounters->nCRC_P           += pAbsCounters->nCRC_P;
      p1dayCounters->nCRCP_P          += pAbsCounters->nCRCP_P;
      p1dayCounters->nCV_P            += pAbsCounters->nCV_P;
      p1dayCounters->nCVP_P           += pAbsCounters->nCVP_P;
      p1dayCounters->nHEC             += pAbsCounters->nHEC;
      p1dayCounters->nIBE             += pAbsCounters->nIBE;
      p1dayCounters->nTotalCells      += pAbsCounters->nTotalCells;
      p1dayCounters->nUserTotalCells  += pAbsCounters->nUserTotalCells;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Update Showtime counters*/
      if( DSL_DRV_PM_CONTEXT(pContext)->bShowtimeProcessingStart )
      {
         pShowtimeCounters->nCRC_P          += pAbsCounters->nCRC_P;
         pShowtimeCounters->nCRCP_P         += pAbsCounters->nCRCP_P;
         pShowtimeCounters->nCV_P           += pAbsCounters->nCV_P;
         pShowtimeCounters->nCVP_P          += pAbsCounters->nCVP_P;
         pShowtimeCounters->nHEC            += pAbsCounters->nHEC;
         pShowtimeCounters->nIBE            += pAbsCounters->nIBE;
         pShowtimeCounters->nTotalCells     += pAbsCounters->nTotalCells;
         pShowtimeCounters->nUserTotalCells += pAbsCounters->nUserTotalCells;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

      /* Update Total counters*/
      pTotalCounters->nCRC_P          += pAbsCounters->nCRC_P;
      pTotalCounters->nCRCP_P         += pAbsCounters->nCRCP_P;
      pTotalCounters->nCV_P           += pAbsCounters->nCV_P;
      pTotalCounters->nCVP_P          += pAbsCounters->nCVP_P;
      pTotalCounters->nHEC            += pAbsCounters->nHEC;
      pTotalCounters->nIBE            += pAbsCounters->nIBE;
      pTotalCounters->nTotalCells     += pAbsCounters->nTotalCells;
      pTotalCounters->nUserTotalCells += pAbsCounters->nUserTotalCells;
   }

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_THRESHOLDS
   #ifdef INCLUDE_DSL_CPE_PM_HISTORY
   #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS
   /* Check for the Channel Counters 15-min and 1-day interval Thresholds Crossing*/
   nErrCode = DSL_DRV_PM_DataPathCountersThresholdsCrossingCheck(
                 pContext, nChannel, nDirection, p15minCounters, p1dayCounters);
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_OPTIONAL_PARAMETERS*/
   #endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_THRESHOLDS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DataPathCountersUpdate, retCode=%d" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

/*
   Function to reset Data Path Counters
*/
DSL_Error_t DSL_DRV_PM_DataPathCountersReset(
   DSL_Context_t *pContext,
   DSL_PM_ResetTypes_t ResetType)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   DSL_int_t hist15minIdx = -1, hist1dayIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   DSL_int_t histShowtimeIdx = -1;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DataPathCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_15MIN(),
                 0, &hist15minIdx);

   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_1DAY(),
                 0, &hist1dayIdx);

   if( hist15minIdx < 0  || hist1dayIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Data Path history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/

#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   nErrCode = DSL_DRV_PM_HistoryItemIdxGet(
                 pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_SHOWTIME(),
                 0, &histShowtimeIdx);

   if( histShowtimeIdx < 0 || nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Data Path showtime history index error!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/

   switch(ResetType)
   {
   case DSL_PM_RESET_CURRENT:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
   /* Reset current 15-min and 1-day values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min[hist15minIdx]),
          0x0, sizeof(DSL_pmDataPathData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day[hist1dayIdx]),
          0x0, sizeof(DSL_pmDataPathData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
   /* Reset current showtime values */
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime[histShowtimeIdx]),
          0x0, sizeof(DSL_pmDataPathData_t));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   break;

   case DSL_PM_RESET_HISTORY:
   case DSL_PM_RESET_ALL:
#ifdef INCLUDE_DSL_CPE_PM_HISTORY
      /* Delete all history intervals*/
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_15MIN() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM data path history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_1DAY() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM data path history 15min delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_15MIN_RECORDS_NUM + 1));
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_1DAY_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min[hist15minIdx]),
            sizeof(DSL_pmDataPathData_t));

         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day[hist1dayIdx]),
            sizeof(DSL_pmDataPathData_t));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec15min[1]),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_15MIN_RECORDS_NUM));
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.rec1day[1]),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_1DAY_RECORDS_NUM));
      }
     
#endif /* #ifdef INCLUDE_DSL_CPE_PM_HISTORY*/
#ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS
      /* Delete all history intervals*/
      nErrCode = DSL_DRV_PM_HistoryDelete(pContext, DSL_DRV_PM_PTR_DATAPATH_HISTORY_SHOWTIME() );
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM data path history showtime delete failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         break;
      }

      if (ResetType == DSL_PM_RESET_ALL)
      {
         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_SHOWTIME_RECORDS_NUM + 1));
      }
      else
      {
         memcpy(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime[0]),
            &(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime[histShowtimeIdx]),
            sizeof(DSL_pmDataPathData_t));

         memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recShowtime[1]),
                0x0,
                sizeof(DSL_pmDataPathData_t) * (DSL_PM_DATAPATH_SHOWTIME_RECORDS_NUM));
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_SHOWTIME_COUNTERS*/
   if( ResetType == DSL_PM_RESET_HISTORY )
      break;

   case DSL_PM_RESET_TOTAL:
#ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS
      memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recTotal),
             0x0,
             sizeof(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recTotal));
#endif /* #ifdef INCLUDE_DSL_CPE_PM_TOTAL_COUNTERS*/
      break;

   default:
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown reset type!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
      
      break;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DataPathCountersReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/

static DSL_Error_t DSL_DRV_PM_FeCountersResetCheck(
   DSL_Context_t *pContext,
   DSL_uint32_t nES)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_LineSecData_t *pPrevLineSecCounters;
   DSL_uint32_t dT = 0;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_PM_FeCountersResetCheck" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   pPrevLineSecCounters = DSL_DRV_PM_PTR_LINE_SEC_COUNTERS_CURR(DSL_FAR_END);

   if( !pPrevLineSecCounters )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - NULL pointer detected!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   /* Get time interval between current and previous FE request*/
   dT = (DSL_DRV_PM_CONTEXT(pContext)->pmThreadFe.nThreadPollTime)/DSL_PM_MSEC; /* sec */

   /* Check the FE TR1 reset condition*/
   if( (nES < pPrevLineSecCounters->nES) &&
       (pPrevLineSecCounters->nES < DSL_PM_COUNTER_ES_MAX_VALUE -dT) )
   {
      /* Far-End counters were resetted due to the TR1 interval*/
      DSL_DRV_PM_CONTEXT(pContext)->bFeTr1Reset = DSL_TRUE;
   }
   else
   {
      /* Clear the FE counters reset indication flag*/
      DSL_DRV_PM_CONTEXT(pContext)->bFeTr1Reset = DSL_FALSE;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_FeCountersResetCheck, retCode=%d"DSL_DRV_CRLF,\
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}


DSL_Error_t DSL_DRV_PM_CountersUpdate(
   DSL_Context_t *pContext,
   const DSL_XTUDir_t nDirection)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t nChannel;
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
   DSL_PM_ChannelData_t  channelData[DSL_CHANNELS_PER_LINE];
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/
#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
   DSL_PM_DataPathData_t dataPathData[DSL_CHANNELS_PER_LINE];
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   DSL_pmLineData_t lineData;

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ATU_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_CountersUpdate" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));


   /* Get current PM module Line counters*/
   nErrCode = DSL_DRV_PM_LineCountersCurrentGet(pContext, nDirection, &lineData);
   if( nErrCode < DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Line Counters current get failed, nDirection=%d!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext), nDirection));
            
      return nErrCode;
   }

   /* Get current PM module Bearer channel dependent counters*/
   for( nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++ )
   {
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Get current Channel counters*/
      nErrCode = DSL_DRV_PM_ChannelCountersCurrentGet(
                    pContext, nChannel, nDirection, &channelData[nChannel]);
      if( nErrCode < DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Channel Counters current get failed, nChannel=%d,nDirection=%d,!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nChannel, nDirection));
            
         return nErrCode;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Get current Data Path counters*/
      nErrCode = DSL_DRV_PM_DataPathCountersCurrentGet(
                    pContext, nChannel, nDirection, &dataPathData[nChannel]);
      if( nErrCode < DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Data Path Counters current get failed, nChannel=%d,nDirection=%d,!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext), nChannel, nDirection));
         return nErrCode;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   }

   /* Check for the FE TR1 reset condition*/
   if( nDirection == DSL_FAR_END )
   {
      nErrCode = DSL_DRV_PM_FeCountersResetCheck(pContext, lineData.sec_fe.nES);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - TR1 reset condition check failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

   /*
      Update all PM module counters
   */
   /* Lock PM module access mutex*/
   nErrCode = DSL_DRV_PM_AccessMutexControl(pContext, nDirection, DSL_TRUE);
   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM Near-End mutex lock failed!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      
      return nErrCode;
   }

   while(1)
   {
      /* Update PM module Line Counters*/
      nErrCode = DSL_DRV_PM_LineCountersUpdate(pContext, nDirection, &lineData);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Line counters update failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         
         break;
      }

      /* Update PM module Bearer channel dependent counters*/
      for( nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++ )
      {
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
         /* Update Channel Counters*/
         nErrCode = DSL_DRV_PM_ChannelCountersUpdate(
                       pContext, nChannel, nDirection, &channelData[nChannel]);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Channel counters update failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            break;
         }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
         /* Update Data Path Counters*/
         nErrCode = DSL_DRV_PM_DataPathCountersUpdate(
                       pContext, nChannel, nDirection, &dataPathData[nChannel]);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Data Path counters update failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            break;
         }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
      }

      break;
   } /* while(1)*/

   /* Unlock PM module access mutex*/
   DSL_DRV_PM_AccessMutexControl(pContext, nDirection, DSL_FALSE);

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_CountersUpdate, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_CountersSave(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint8_t i = 0, nChannel = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_CountersSave" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   DSL_CHECK_POINTER(pContext, DSL_DRV_PM_CONTEXT(pContext));
   DSL_CHECK_ERR_CODE();

   for (i = 0; i < 2; i++)
   {
      /* Get Line Counters from the FW*/
      nErrCode = DSL_DRV_PM_LineCountersCurrentGet(
                    pContext,
                    i == 0 ? DSL_NEAR_END : DSL_FAR_END,
                    &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->LineCounters));

      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Line Counters save failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return nErrCode;
      }

      /* Get current PM module Bearer channel dependent counters*/
      for( nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++ )
      {
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
         /* Get Channel Counters from the FW*/
         nErrCode = DSL_DRV_PM_ChannelCountersCurrentGet(
                       pContext, nChannel,
                       i == 0 ? DSL_NEAR_END : DSL_FAR_END,
                       i == 0 ?
                       &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->ChannelCounters.data_ne[nChannel]):
                       &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->ChannelCounters.data_fe[nChannel]));

         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Channel Counters save failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }
#endif /* INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
         nErrCode = DSL_DRV_PM_DataPathCountersCurrentGet(
                       pContext, nChannel,
                       i == 0 ? DSL_NEAR_END : DSL_FAR_END,
                       i == 0 ?
                       &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->DataPathCounters.data_ne[nChannel]):
                       &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->DataPathCounters.data_fe[nChannel]));

         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Data Path Counters save failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }
#endif /* INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
      }
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_CountersSave, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_CountersRestore(
   DSL_Context_t *pContext,
   DSL_uint32_t fwUnavailableTime)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_boolean_t bFwReLoaded = DSL_FALSE;
   DSL_uint8_t nChannel, i = 0;
   DSL_XTUDir_t nDirection;
#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
   DSL_PM_ChannelData_t *pChannelCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
   DSL_PM_LineInitData_t *pInitCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/
#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
   DSL_PM_DataPathData_t *pDataPathCounters = DSL_NULL;
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   DSL_PM_LineSecData_t *pSecCounters = DSL_NULL;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_CountersRestore" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL || DSL_DRV_PM_CONTEXT(pContext)->bInit == DSL_FALSE )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM Counters restore failed, PM module not initializaed yet!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   DSL_CTX_READ_SCALAR(pContext, nErrCode, bFwReLoaded, bFwReLoaded);

   /* Restore all PM module counters*/
   if( bFwReLoaded )
   {
      /* Restart PM device specific stuff*/
      nErrCode = DSL_DRV_PM_DEV_Restart(pContext);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM device specific initialization failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
      /* Restore Channel Counters*/
      for( i = 0; i < 2; i++ )
      {
         nDirection = i ? DSL_NEAR_END : DSL_FAR_END;

         /* Update Channel Counters for the Near-End direction*/
         for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
         {
            /* Get Pointer to the Channel Saved Counters*/
            pChannelCounters =
               nDirection == DSL_NEAR_END ? 
               &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->ChannelCounters.data_ne[nChannel]):
               &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->ChannelCounters.data_fe[nChannel]);

            /* Set Channel counters in the FW*/
            nErrCode = DSL_DRV_PM_DEV_ChannelCountersSet(
               pContext, nChannel, nDirection, pChannelCounters);

            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - PM %s Channel(%d) Counters Set failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext), nDirection == DSL_NEAR_END ? "NE" : "FE", nChannel));
                  
               break;
            }
         }
      }

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Channel Counters restore failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/


      /* Restore Line Sec counters*/
      for( i = 0; i < 2; i++ )
      {
         nDirection = i ? DSL_NEAR_END : DSL_FAR_END;

         /* Get Pointer to the Line Sec Saved Counters, NE direction*/
         pSecCounters =
            nDirection == DSL_NEAR_END ? 
               &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->LineCounters.sec_ne):
               &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->LineCounters.sec_fe);

         /* Take into account FW unavailable time*/
         pSecCounters->nUAS += fwUnavailableTime;

         /* Set Line Sec counters in the FW*/
         nErrCode = DSL_DRV_PM_DEV_LineSecCountersSet(pContext, nDirection, pSecCounters);
         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
              (pContext, "DSL[%02d]: ERROR - PM %s Line Sec Counters Set failed!"DSL_DRV_CRLF,
              DSL_DEV_NUM(pContext), nDirection == DSL_NEAR_END ? "NE" : "FE"));
              
            break;
         }
      }

      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Line Sec Counters restore failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }


      /* Restore Line Init Counters*/
#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
      /* Get Pointer to the Line Init Total Counters*/
      pInitCounters = &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->LineCounters.init);

      /* Set Line Init counters in the FW*/
      nErrCode = DSL_DRV_PM_DEV_LineInitCountersSet(pContext, pInitCounters);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM Line Init Counters restore failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

      /* Restore Data Path Counters*/
#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
      /* Update Channel Counters for the Near-End direction*/
      for (nChannel = 0; nChannel < DSL_CHANNELS_PER_LINE; nChannel++)
      {
         /* Get Pointer to the Data Path Total Counters, NE direction*/
         pDataPathCounters =
            &(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump->DataPathCounters.data_ne[nChannel]);

         /* Restore Data Path counters, NE direction*/
         nErrCode = DSL_DRV_PM_DEV_DataPathCountersSet(
            pContext, 0, DSL_NEAR_END, pDataPathCounters);

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PM NE Data Path Counters restore failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
         }
      }
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/
   }

   DSL_DEBUG( DSL_DBG_MSG,(pContext, "DSL[%02d]: OUT - "
      "DSL_DRV_PM_CountersRestore, retCode=%d"DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_InternalStateReset(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_InternalStateReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   if( DSL_DRV_PM_CONTEXT(pContext) == DSL_NULL || DSL_DRV_PM_CONTEXT(pContext)->bInit == DSL_FALSE )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - PM module not initializaed yet!"DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_ERROR;
   }

   memset(DSL_DRV_PM_CONTEXT(pContext)->pCountersDump, 0x0, sizeof(DSL_PM_CountersDump_t));

   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recAbs.sec_fe), 0x0, sizeof(DSL_PM_LineSecData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recAbs.sec_ne), 0x0, sizeof(DSL_PM_LineSecData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recCurr.sec_fe), 0x0, sizeof(DSL_PM_LineSecData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.recCurr.sec_ne), 0x0, sizeof(DSL_PM_LineSecData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->lineCounters.auxData), 0x0, sizeof(DSL_pmLineAuxData_t));   

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recAbs), 0x0, sizeof(DSL_pmChannelData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->channelCounters.recCurr), 0x0, sizeof(DSL_pmChannelData_t));   
#endif /* INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recAbs), 0x0, sizeof(DSL_pmDataPathData_t));
   memset(&(DSL_DRV_PM_CONTEXT(pContext)->pCounters->dataPathCounters.recCurr), 0x0, sizeof(DSL_pmDataPathData_t));   
#endif /* INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_InternalStateReset" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_Suspend(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   
   if (DSL_DRV_PM_CONTEXT(pContext) != DSL_NULL && 
       DSL_DRV_PM_CONTEXT(pContext)->bPmLock != DSL_TRUE)
   {
      if (DSL_DRV_PM_CONTEXT(pContext)->bInit)
      {
         /* Lock PM module processing*/
         nErrCode = DSL_DRV_PM_Lock(pContext);
         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Couldn't lock PM module!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }

         /* Check if FW polling is allowed*/
         if (DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid)
         {
            /* Save all PM counters*/
            nErrCode = DSL_DRV_PM_CountersSave(pContext);

            if (nErrCode != DSL_SUCCESS)
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - PM counters save failed!"DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));

               /* Reset Internal PM state*/
               DSL_DRV_PM_InternalStateReset(pContext);
            }
         }

         nErrCode = DSL_DRV_PM_DEV_Suspend(pContext);
         if (nErrCode != DSL_SUCCESS)
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PM module device suspend failed!"DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }
      }
   }

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_Resume(
   DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t fwDwnldTime = 0;
      
   if (DSL_DRV_PM_CONTEXT(pContext) != DSL_NULL &&
       DSL_DRV_PM_CONTEXT(pContext)->bPmLock)
   {
      /* Set FW unavailable time*/
      DSL_CTX_READ_SCALAR(pContext, nErrCode, nFwUnavailableTime, fwDwnldTime);

      DSL_DEBUG( DSL_DBG_MSG,
            (pContext, "DSL[%02d]: Autoboot restore counters..."DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

      /* Restore PM module counters*/
      nErrCode = DSL_DRV_PM_CountersRestore(pContext, fwDwnldTime);
      if( nErrCode != DSL_SUCCESS )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM counters restore failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* Reset FW unavailable time*/
      DSL_CTX_WRITE_SCALAR(pContext, nErrCode, nFwUnavailableTime, 0x0);

      DSL_DEBUG( DSL_DBG_MSG,
            (pContext, "DSL[%02d]: Autoboot unlock PM module..."DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));

      /* Unlock PM module processing*/
      nErrCode = DSL_DRV_PM_UnLock(pContext);
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM module unlock failed!"DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }
   }

   return nErrCode;
}

/** @} DRV_DSL_CPE_PM */

#endif
