/****************************************************************************
**   FILE NAME     : atm_autosearch.c
**   PROJECT       : Bridge and Routing mode
**   MODULES       : ATM Layer
**   SRC VERSION   : V1.0
**   DATE          : 14-05-2008
**   AUTHOR        : Richard Hsu
**   COPYRIGHT     : Copyright (c) 2006
**                   Infineon Technologies AG
**                   Am Campeon 1-12, 85579 Neubiberg, Germany
**
**   Any use of this software is subject to the conclusion of a respective
**   License agreement. Without such a License agreement no rights to the
**   software are granted
**  Version Control Section  **
*****************************************************************************/

#include <linux/atm_autosearch.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <linux/if.h>
#include <linux/netdevice.h>
#include <linux/netlink.h>
#include <net/sock.h>
#include <asm/uaccess.h>

unsigned char as_dhcp_buffer[400];
int stop_autosearch_init_thread = 0;
static wait_queue_head_t wait_queue_init_thread;
//static struct completion autosearch_init_thread_exit;

extern int g_showtime;
struct sk_buff *ash_skb1;
int padi_detected = 0;
int dhcp_detected = 0;
int as_start_polling = 0;
int Autosearch_PVC_creation_OK = 0;
int Autosearch_PVC_modification_OK = 0;
int as_finished = 0;
unsigned int as_vpi;
unsigned int as_vci;
long trigger_time = 0;
char f_AUTOSEARCH_BRIDGE_MODE = 0;
char f_AUTOSEARCH_ROUTER_MODE = 0;
char f_FOUND_EXISTED_PVC = 0;
char f_PVC_CREATION_PART1_OK = 0;
char f_PVC_CREATION_PART2_OK = 0;

struct sock *nl_sk = NULL;

static void ash_krcv(struct sock *sk, int len)
{
    struct sk_buff *skb;
    struct nlmsghdr *nlh = NULL;
    int err;
    char *status_str;
    u32 pid;
    unsigned char as_msg[128];

    skb = skb_recv_datagram(nl_sk, 0, 0, &err);


    if (skb != NULL)
	nlh = (struct nlmsghdr *) skb->data;
    //printk("%s: received netlink message payload:%s\n",__FUNCTION__, NLMSG_DATA(nlh));
    //printk("hsur: received netlink message from user space:%s\n", NLMSG_DATA(nlh));
    status_str = NLMSG_DATA(nlh);

    switch (status_str[0]) {
    case FOUND_EXISTED_PVC:
	f_FOUND_EXISTED_PVC = 1;
	Autosearch_PVC_creation_OK = 1;

	break;

    case AUTOSEARCH_BRIDGE_MODE:
	f_AUTOSEARCH_BRIDGE_MODE = 1;
	as_msg[0] = 0;
	as_msg[1] = '\0';
	strcpy(NLMSG_DATA(nlh), as_msg);
	printk("in kernel: Autosearch Bridge mode \n");
	break;

    case AUTOSEARCH_ROUTER_MODE:
	f_AUTOSEARCH_ROUTER_MODE = 1;
	as_msg[0] = 0;
	as_msg[1] = '\0';
	strcpy(NLMSG_DATA(nlh), as_msg);
	printk("in kernel:  Autosearch Router mode \n");
	// We dont; need to check DHCP/PADI receive flag , CPE will send the DHCP/PADI out
	dhcp_detected = 1;
	padi_detected = 1;
	//trigger_time

	break;

    case PVC_CREATION_PART1_OK:
	as_msg[0] = PVC_CREATION_PART1_OK;
	as_msg[1] = '\0';
	strcpy(NLMSG_DATA(nlh), as_msg);
	Autosearch_PVC_creation_OK = 1;
	f_PVC_CREATION_PART1_OK = 1;
	break;

    case PVC_CREATION_PART2_OK:
	as_msg[0] = PVC_CREATION_PART2_OK;
	as_msg[1] = '\0';
	strcpy(NLMSG_DATA(nlh), as_msg);
	Autosearch_PVC_creation_OK = 1;
	f_PVC_CREATION_PART2_OK = 1;
	break;

    case AUTOSEARCH_POLLING:
	if (!as_finished && (trigger_time != 0))	//&& (jiffies - trigger_time > 1500))
	{
	    if (f_FOUND_EXISTED_PVC && (jiffies - trigger_time > 500)) {
		as_msg[0] = FOUND_EXISTED_PVC_FAIL;
		as_msg[1] = '\0';
		strcpy(NLMSG_DATA(nlh), as_msg);
		printk("EXISTED PVC FAILED\n");
		trigger_time = 0;
		Autosearch_PVC_creation_OK = 0;
		f_FOUND_EXISTED_PVC = 0;
	    } else if (jiffies - trigger_time > 1500) {
		if (f_PVC_CREATION_PART1_OK) {
		    as_msg[0] = AUTOSEARCH_PART1_FAIL;
		    f_PVC_CREATION_PART1_OK = 0;
		    printk("PART 1 FAILED\n");
		} else {
		    as_msg[0] = AUTOSEARCH_PART2_FAIL;
		    f_PVC_CREATION_PART2_OK = 0;
		    printk("PART 2 FAILED\n");
		}
		as_msg[1] = '\0';
		strcpy(NLMSG_DATA(nlh), as_msg);

		trigger_time = 0;
		Autosearch_PVC_creation_OK = 0;
	    }
	}

	if (as_finished) {
	    as_msg[0] = AUTOSEARCH_PVC_DONE;
	    if (as_vpi == 0) {
		as_msg[1] = PVC_SPECIAL_CASE;
		as_msg[2] = 99;
	    } else {
		as_msg[1] = PVC_NORMAL_CASE;
		as_msg[2] = as_vpi;
	    }

	    if (as_vci == 0) {
		as_msg[3] = PVC_SPECIAL_CASE;
		as_msg[4] = 99;
		as_msg[5] = 99;
	    } else {
		as_msg[3] = PVC_NORMAL_CASE;
		as_msg[4] = as_vci;	// lower byte first   
		as_msg[5] = as_vci >> 8;
	    }
	    as_msg[6] = '\0';
	    strcpy(NLMSG_DATA(nlh), as_msg);

	}
	break;

    default:
	strcpy(NLMSG_DATA(nlh), "");
	break;
    }

    pid = nlh->nlmsg_pid;	/*pid of sending process */
    NETLINK_CB(skb).groups = 0;	/* not in mcast group */
    NETLINK_CB(skb).pid = 0;	/* from kernel */
    NETLINK_CB(skb).dst_pid = pid;
    NETLINK_CB(skb).dst_groups = 0;	/* unicast */

    netlink_unicast(nl_sk, skb, pid, MSG_DONTWAIT);

}

void autosearch_netlink()
{

    nl_sk = netlink_kernel_create(NETLINK_IFX_ASH, ash_krcv);
    if (!nl_sk) {
	printk(KERN_ERR
	       "open autosearch kernel socket failed! nl_sk is %x\n",
	       nl_sk);
	return -1;
    }
    return 0;
}


static int autosearch_init_thread(void *data)
{
    struct task_struct *tsk = current;
    int i = 0;
    unsigned long arg;


    init_waitqueue_head(&wait_queue_init_thread);
    daemonize();
    sigfillset(&tsk->blocked);
    autosearch_netlink();
    as_start_polling = 1;


    while (!stop_autosearch_init_thread) {
	if (as_finished) {
	    stop_autosearch_init_thread = 1;
	    printk("Autosearch done in kernel space: VPI: %d VCI: %d\n",
		   as_vpi, as_vci);
	}
	if (padi_detected) {
	    //stop_autosearch_init_thread = 1; 
	    //printk("received PADI now \n");
	}
	if (dhcp_detected) {
	    //stop_autosearch_init_thread = 1;
	    //printk("received DHCP request  now \n");

	}
	if (Autosearch_PVC_creation_OK && !Autosearch_PVC_modification_OK) {
	    //ash_addif();
	    Autosearch_PVC_modification_OK = 1;
	}

	interruptible_sleep_on_timeout(&wait_queue_init_thread, 5);
	i++;
	if (i == 500) {
	    //printk("Autosearch thread is still running\n");
	    //printk("trigger_time is %d\n",trigger_time);  
	    //i = 0;
	}
    }
    as_start_polling = 0;
    //complete_and_exit (&autosearch_init_thread_exit, 0);
}






/*
 * ===========================================================================
 *                           EXPORTED FUNCTIONS
 * ===========================================================================
 */
/********************************************************************
*  Function Name  : main 
*  Description    : This is the main for autosearch feature 
*  Input Values   :  
*  Output Values  :  
*  Return Value   :
*  Notes          : 
*********************************************************************/
int atm_autosearch_init()
{
    printk("ATM_autosearch_init thread begins\n");
    kernel_thread(autosearch_init_thread, NULL,
		  CLONE_FS | CLONE_FILES | CLONE_SIGHAND);
}




// Not used 

static int ash_delif()
{


    struct net_device *nas[MAX_PVC_TRYING];
    struct net_device *br0;
    struct net_device *eth0;
    struct net_bridge *br;
    int i;
    eth0 = dev_get_by_name("eth0");
    nas[0] = dev_get_by_name("nas0");
    nas[1] = dev_get_by_name("nas1");
    nas[2] = dev_get_by_name("nas2");
    nas[3] = dev_get_by_name("nas3");
    nas[4] = dev_get_by_name("nas4");
    nas[5] = dev_get_by_name("nas5");
    nas[6] = dev_get_by_name("nas6");
    nas[7] = dev_get_by_name("nas7");
    nas[8] = dev_get_by_name("nas8");
    nas[9] = dev_get_by_name("nas9");
    nas[10] = dev_get_by_name("nas10");
    nas[11] = dev_get_by_name("nas11");
    nas[12] = dev_get_by_name("nas12");
    nas[13] = dev_get_by_name("nas13");

    br0 = dev_get_by_name("br0");
    br = br0->priv;
    for (i = 0; i < MAX_PVC_TRYING; i++) {
	if (nas[i] != NULL) {
	    //atomic_set(&nas[i]->refcnt, 1);
	    br_del_if(br, nas[i]);
	} else {
	    //printk("!!!! nas%d pvc error !!!!!!! \n",i );
	}
    }

}

static int ash_addif()
{

    struct net_device *nas[MAX_PVC_TRYING];
    struct net_device *br0;
    struct net_device *eth0;
    struct net_bridge *br;
    int i;
    eth0 = dev_get_by_name("eth0");
    nas[0] = dev_get_by_name("nas0");
    nas[1] = dev_get_by_name("nas1");
    nas[2] = dev_get_by_name("nas2");
    nas[3] = dev_get_by_name("nas3");
    nas[4] = dev_get_by_name("nas4");
    nas[5] = dev_get_by_name("nas5");
    nas[6] = dev_get_by_name("nas6");
    nas[7] = dev_get_by_name("nas7");
    nas[8] = dev_get_by_name("nas8");
    nas[9] = dev_get_by_name("nas9");
    nas[10] = dev_get_by_name("nas10");
    nas[11] = dev_get_by_name("nas11");
    nas[12] = dev_get_by_name("nas12");
    nas[13] = dev_get_by_name("nas13");

    br0 = dev_get_by_name("br0");
    br = br0->priv;
    for (i = 0; i < MAX_PVC_TRYING; i++) {
	if (nas[i] != NULL) {
	    printk("nas[%d] dev is %x \n", i, nas[i]);
	    nas[i]->flags |= (IFF_RUNNING | IFF_UP);	// up the if
	    nas[i]->dev_addr[0] = (unsigned char) eth0->dev_addr[0];
	    nas[i]->dev_addr[1] = (unsigned char) eth0->dev_addr[1];
	    nas[i]->dev_addr[2] = (unsigned char) eth0->dev_addr[2];
	    nas[i]->dev_addr[3] = (unsigned char) eth0->dev_addr[3];
	    nas[i]->dev_addr[4] = (unsigned char) eth0->dev_addr[4];
	    nas[i]->dev_addr[5] =
		(unsigned char) eth0->dev_addr[5] + 1 + i;
	    nas[i]->dev_addr[6] =
		(unsigned char) eth0->dev_addr[6] + 1 + i;
	    br_add_if(br, nas[i]);
	} else {
	    printk("!!!! nas%d pvc error !!!!!!! \n", i);
	}
    }



}
