/******************************************************************************
**
** FILE NAME    : amazon_se.c
** PROJECT      : Amazon_se
** MODULES      : NOR Flash
**
** DATE         : 
** AUTHOR       : 
** DESCRIPTION  : NOR Flash MTD Driver (Mapping)
** COPYRIGHT    :       Copyright (c) 2006
**                      Infineon Technologies AG
**                      Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date        $Author         $Comment
** 12/04/2007	Teh Kok How	Version 2.0
*******************************************************************************/

/*
 * Handle mapping of the flash memory access routines
 * on Amazon_se  based devices.
 *
 * Copyright(C) 2004 peng.liu@infineon.com
 *
 * This code is GPLed
 *
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>

#include <linux/init.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/cfi.h>
#include <asm/amazon_se/amazon_se.h>


#if defined(CONFIG_PCI) && defined(CONFIG_AMAZON_SE_CHIP_A11) && defined(CONFIG_AMAZON_SE_EBU_PCI_SOFTWARE_ARBITOR)
  #define EBU_PCI_SOFTWARE_ARBITOR
#endif

#ifdef EBU_PCI_SOFTWARE_ARBITOR
  #define AMAZON_SE_PCI_REG32( addr )          (*(volatile u32 *)(addr))
#endif


#define FLASH_BANK_MAX 1
#if 0
#define AMAZON_SE_MTD_DMSG(fmt, args...) printk( KERN_ERR  "%s: " fmt,__FUNCTION__, ## args)
#else
#define AMAZON_SE_MTD_DMSG(fmt, args...)
#endif
#define AMAZON_SE_MTD_EMSG(fmt, args...) printk( KERN_ERR  "%s: " fmt,__FUNCTION__, ## args)
/* trivial struct to describe partition information */
struct mtd_part_def
{
        int nums;
        unsigned char *type;
        struct mtd_partition* mtd_part;
};

/* static struct mtd_info *mymtd; */
static struct mtd_info* mtd_banks[FLASH_BANK_MAX];
static struct map_info* map_banks[FLASH_BANK_MAX];
static struct mtd_part_def part_banks[FLASH_BANK_MAX];
static unsigned long num_banks;
static unsigned long start_scan_addr;
static int probing = 0;

#define AMAZON_SE_MTD_REG32( addr )                  (*(volatile u32 *)(addr))
static unsigned long irq_flags;
static void inline enable_ebu(void)
{
#ifdef EBU_PCI_SOFTWARE_ARBITOR
    u32 temp_buffer;

    /* drive CFRAME_N/GPIO7 to 0 */
  #if 0
    *AMAZON_SE_GPIO_P0_OUT &= (~(1<<7));
    *AMAZON_SE_GPIO_P0_OD |=  ((1<<7));
    *AMAZON_SE_GPIO_P0_DIR |=  ((1<<7));
    *AMAZON_SE_GPIO_P0_ALTSEL1 &= (~(1<<7));
    *AMAZON_SE_GPIO_P0_ALTSEL0 &= (~(1<<7));
  #endif
    //  printk("\nenable_ebu\n");

    //Delay before enabling ebu
    for(temp_buffer=0; temp_buffer<9999;temp_buffer++);

    /* disable int/ext pci request */
    temp_buffer = AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG);
    temp_buffer &= ~0x0C00;
    AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG) = temp_buffer;
    temp_buffer |= 0x3300;
    AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG) = temp_buffer;

    /* poll for pci bus idle */
    temp_buffer = AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG);
//  printk("PCI_CR_PC_ARB_REG1: %08X",temp_buffer);
    while ((temp_buffer & 0x300000) != 0x300000) {
//      printk("PCI_CR_PC_ARB_REG: %08X",temp_buffer);
        temp_buffer = AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG);
    };

    AMAZON_SE_PCI_REG32(0xBE105400) = 0;

    /* unmask CFRAME_MASK */
    /* changing PCI's Config Space via internal path */
    /* might need to change to external path */
    AMAZON_SE_PCI_REG32(0xBE105430) = 0x00000103;
    temp_buffer = AMAZON_SE_PCI_REG32(0xB700006C);
    temp_buffer &= (~0x00000001);
    AMAZON_SE_PCI_REG32(0xB700006C) = temp_buffer;
    AMAZON_SE_PCI_REG32(0xBE105430) = 0x01000103;
    temp_buffer = AMAZON_SE_PCI_REG32(0xB700006C); // do dummy read

  #if 0
    printk("\nwaiting ebu ownnership");
    while ((AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG) & 0x400000) != 0) {
//      *AMAZON_SE_GPIO_P0_OUT |= ((1<<7));
        printk(".");
    }
//  *AMAZON_SE_GPIO_P0_OUT &= (~(1<<7));
    printk("done\n");
  #endif
#endif
}

static void inline disable_ebu(void)
{
#ifdef EBU_PCI_SOFTWARE_ARBITOR
    u32 temp_buffer;
    //printk("\ndisable_ebu\n");
    //Delay before enabling ebu
    for(temp_buffer=0; temp_buffer<9999;temp_buffer++);

     AMAZON_SE_PCI_REG32(0xBE105400) = 0x2;
    /* unmask CFRAME_MASK */
    /* changing PCI's Config Space via internal path */
    /* might need to change to external path */
    AMAZON_SE_PCI_REG32(0xbE105430) = 0x00000103;
    temp_buffer = AMAZON_SE_PCI_REG32(0xB700006C);
    temp_buffer |= 0x00000001;
    AMAZON_SE_PCI_REG32(0xB700006C) = temp_buffer;
    AMAZON_SE_PCI_REG32(0xbE105430) = 0x01000103;

    /* enable int/ext pci request */
    temp_buffer = AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG);
    temp_buffer &= (~0x3300);
    temp_buffer |= 0x0C00;
    AMAZON_SE_PCI_REG32(PCI_CR_PC_ARB_REG) = temp_buffer;
#endif
}

__u8 amazon_se_read8(struct map_info *map, unsigned long ofs)
{
        u8 temp;
        enable_ebu();
        temp = *((__u8 *)(map->map_priv_1 + ofs));
        disable_ebu();
        AMAZON_SE_MTD_DMSG("8: [%p + %x] ==> %x \n",map->map_priv_1, ofs, temp);
        return temp;
}

__u16 amazon_se_read16(struct map_info *map, unsigned long ofs)
{
 /* Modify the address offset to account the EBU address swap.
  * Needed only for probing.
  */
        u16 temp;
        if (probing)
                ofs ^= 2;
        enable_ebu();
        temp = *((__u16 *)(map->map_priv_1 + ofs));
        disable_ebu();
        AMAZON_SE_MTD_DMSG("16: [%p + %x] ==> %x \n",map->map_priv_1, ofs, temp);
        return temp;
}

__u32 amazon_se_read32(struct map_info *map, unsigned long ofs)
{
        u32 temp;
        enable_ebu();
        temp = *((__u32 *)(map->map_priv_1 + ofs));
        disable_ebu();
        AMAZON_SE_MTD_DMSG("32: [%p + %x] ==> %x \n",map->map_priv_1, ofs, temp);
        return temp;
}

void amazon_se_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
        AMAZON_SE_MTD_DMSG("from:%x to:%x len:%d",from,to,len);
#ifdef CONFIG_AMAZON_SE
//work around for EBU access that causes unaligned access exception
        u8 *p;
        u8 *to_8;

        enable_ebu();
        from = (unsigned long) (from + map->map_priv_1);
        if  ( (((unsigned long)to) & 3) == (from & 3) ) {
                memcpy_fromio(to, (void *)from, len);
        }else {
                p = (u8 *) (from);
                to_8 = (u8 *) (to);
                while(len--){
                        *to_8++ = *p++;
                }
        }
        disable_ebu();
#else
        memcpy_fromio(to, (void *)(map->map_priv_1 + from), len);
#endif

}

void amazon_se_write8(struct map_info *map, __u8 d, unsigned long adr)
{
        enable_ebu();
        *((__u8 *)(map->map_priv_1 + adr)) = d;
        disable_ebu();
        AMAZON_SE_MTD_DMSG("8: [%p + %x] <== %x \n",map->map_priv_1, adr, d);
}

void amazon_se_write16(struct map_info *map, __u16 d, unsigned long adr)
{
 /* Modify the address offset to account the EBU address swap.
  * Needed only for probing.
  */
        if (probing)
                adr ^= 2;
        enable_ebu();
        *((__u16 *)( map->map_priv_1 + adr)) = d;
        disable_ebu();
        AMAZON_SE_MTD_DMSG("16: [%p + %x] <== %x \n",map->map_priv_1, adr, d);
}

void amazon_se_write32(struct map_info *map, __u32 d, unsigned long adr)
{
        enable_ebu();
        *((__u32 *)(map->map_priv_1 + adr)) = d;
        disable_ebu();
        AMAZON_SE_MTD_DMSG("32: [%p + %x] <== %x \n",map->map_priv_1, adr, d);
}

void amazon_se_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
        AMAZON_SE_MTD_DMSG("from:%x to:%x len:%d",from,to,len);
        enable_ebu();
        memcpy_toio((void *)(map->map_priv_1 + to), from, len);
        disable_ebu();
}

struct map_info amazon_se_map = {
        name:           "AMAZON_SE",
        size:     0x2000,
        buswidth:       2,
        read8:          amazon_se_read8,
        read16:         amazon_se_read16,
        read32:         amazon_se_read32,
        copy_from:      amazon_se_copy_from,
        write8:         amazon_se_write8,
        write16:        amazon_se_write16,
        write32:        amazon_se_write32,
        copy_to:        amazon_se_copy_to
};

/*
 * Here are partition information for all known Amazon_se series devices.
 * See include/linux/mtd/partitions.h for definition of the mtd_partition
 * structure.
 */
#if 0 //Mark
#ifdef CONFIG_MTD_PARTITIONS
#if (CONFIG_MTD_AMAZON_SE_FLASH_SIZE == 2)
	#ifdef CONFIG_IFX_ADSL_FIRMWARE_IN_ROOTFS
		#define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x001F0000
	#else
		#define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x001C0000
	#endif
		
#elif (CONFIG_MTD_AMAZON_SE_FLASH_SIZE == 4)
	#ifdef CONFIG_IFX_ADSL_FIRMWARE_IN_ROOTFS
		#define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x003F0000
	#else
		#define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x003C0000
	#endif
#else
	#error  "Configure Amazon_SE MTD flash size first!!"
#endif
#endif /* CONFIG_MTD_PARTITIONS */
#else

#ifdef CONFIG_MTD_PARTITIONS
#if (CONFIG_MTD_AMAZON_SE_FLASH_SIZE == 2)
                #define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x001C0000

#elif (CONFIG_MTD_AMAZON_SE_FLASH_SIZE == 4)
                #define AMAZON_SE_SYSTEM_PARTITION_SIZE    0x003C0000
#else
        #error  "Configure Amazon_SE MTD flash size first!!"
#endif
#endif /* CONFIG_MTD_PARTITIONS */

#endif //Mark
/* partition definition for first flash bank
 * also ref. to "drivers/char/flash_config.c"
 */
#if 0 //Mark
#ifdef CONFIG_IFX_ADSL_FIRMWARE_IN_ROOTFS
static struct mtd_partition amazon_se_partitions[] = {
	{
		name:	"U-Boot",
		offset:	0x00000000,
		size:	0x00010000,
	},
	{
		name:	"RootFS,Kernel,Data,Environment",
		offset:	0x00010000,
		size:	AMAZON_SE_SYSTEM_PARTITION_SIZE,
	},
};
#else
static struct mtd_partition amazon_se_partitions[] = {
	{
		name:	"U-Boot",
		offset:	0x00000000,
		size:	0x00010000,
	},
	{
		name:	"Firmware",
		offset:	0x00010000,
		size:	0x00030000,
	},
	{
		name:	"RootFS,Kernel,Data,Environment",
		offset:	0x00040000,
		size:	AMAZON_SE_SYSTEM_PARTITION_SIZE,
	},
};
#endif /* CONFIG_IFX_ADSL_FIRMWARE_IN_ROOTFS */
#else //Mark
#define _1K             1024
#define _1M             (_1K * _1K)
#define _64K            (64 * _1K)              //64K
#define _2M             (2 * _1M)               //2M
#define BASEOFF         0x00000000
#define BOOT_S          _64K
#define ENV_MAC_S       _64K
#define DPF_S           _64K
#define NVRAM_S         _64K
#define KERNEL_S        (512 * _1K)             //512K
#define ROOTFS_S        (_2M - (BOOT_S + ENV_MAC_S + DPF_S + NVRAM_S + KERNEL_S))

static struct mtd_partition amazon_se_partitions[] = {
        {
                name:   "U-Boot",
                offset: BASEOFF,
                size:   BOOT_S,
        },
        {
                name:   "ENV_MAC",
                offset: BASEOFF + BOOT_S,
                size:   ENV_MAC_S,
        },
        {
                name:   "DPF",
                offset: BASEOFF + BOOT_S + ENV_MAC_S,
                size:   DPF_S,
        },

        {
                name:   "NVRAM",
                offset: BASEOFF + BOOT_S + ENV_MAC_S + DPF_S,
                size:   NVRAM_S,
        },

        {
                name:   "RootFS_DPUMP",
                offset: BASEOFF + BOOT_S + ENV_MAC_S + DPF_S + NVRAM_S,
                size:   ROOTFS_S,
        },

        {
                name:   "Kernel",
                offset: BASEOFF + BOOT_S + ENV_MAC_S + DPF_S + NVRAM_S + ROOTFS_S,
                size:   KERNEL_S,
        },

        {       //for easy web upgrade
                name:   "ROOTFS_KERNEL",
                offset: BASEOFF + BOOT_S + ENV_MAC_S + DPF_S + NVRAM_S,
                size:   ROOTFS_S+KERNEL_S,
        },

};
#endif //Mark
#define NB_OF(x)  (sizeof(x)/sizeof(x[0]))

unsigned long flash_start, flash_size;

int __init init_amazon_se_mtd(void)
{
        int idx = 0, ret = 0;
        unsigned long mtd_size = 0;

        /* setting EBU */
	*AMAZON_SE_EBU_CON &= 0x1FFFFFFF;
	*AMAZON_SE_EBU_CON |= (2 << AMAZON_SE_EBU_CON_EBU_TA_SHIFT);

	*AMAZON_SE_EBU_CON0 &= ~AMAZON_SE_EBU_CON0_WRDIS;
	*AMAZON_SE_EBU_CON0 &= ~AMAZON_SE_EBU_CON0_CMULT_MASK;
	*AMAZON_SE_EBU_CON0 |= (1 << AMAZON_SE_EBU_CON0_CMULT_SHIFT);
// 804031:fchang.removed
//#if 0
// 804031:fchang.added
#if 1

	*AMAZON_SE_EBU_CON0 &= ~0x700;
	*AMAZON_SE_EBU_CON0 |= 0x200;
	*AMAZON_SE_EBU_CON0 &= ~0xC0;
	*AMAZON_SE_EBU_CON0 |= 0x80;
	*AMAZON_SE_EBU_CON0 &= ~0x30;
	*AMAZON_SE_EBU_CON0 |= 0x20;
	*AMAZON_SE_EBU_CON0 &= ~0xC;
	*AMAZON_SE_EBU_CON0 |= 0x8;
#endif
	printk(KERN_INFO "[%s %d]: AMAZON_SE_EBU_CON %#x, AMAZON_SE_EBU_CON0 %#x\n", __func__, __LINE__, *AMAZON_SE_EBU_CON, *AMAZON_SE_EBU_CON0);
        /* GPIO: address23 */
        //*AMAZON_SE_GPIO_P1_DIR = ((*AMAZON_SE_GPIO_P1_DIR) | 0x80); /* P1.7 output*/
        //*AMAZON_SE_GPIO_P1_ALTSEL0 = ((*AMAZON_SE_GPIO_P1_ALTSEL0) & (~0x80)); /* P1.7 ALT:00*/
        //*AMAZON_SE_GPIO_P1_ALTSEL1 = ((*AMAZON_SE_GPIO_P1_ALTSEL1) & (~0x80));

        /* request maximum flash size address space */
        flash_start &= (~0xa0000000);
        start_scan_addr = (unsigned long)ioremap(flash_start, flash_size);
        if (!start_scan_addr) {
                AMAZON_SE_MTD_EMSG ("Failed to ioremap address:0x%x\n",flash_start);
                return -EIO;
        }
        AMAZON_SE_MTD_EMSG("start_scan_addr: %8x\n",start_scan_addr);

        for(idx = 0 ; idx < FLASH_BANK_MAX ; idx++) {
                if(mtd_size >= flash_size)
                        break;

                printk ("%s: chip probing count %d\n",
                        __FUNCTION__, idx);

                map_banks[idx] = (struct map_info *)kmalloc(sizeof(struct map_info), GFP_KERNEL);
                if(map_banks[idx] == NULL) {
                        ret = -ENOMEM;
                        goto error_mem;
                }
                memset((void *)map_banks[idx], 0, sizeof(struct map_info));
                map_banks[idx]->name = (char *)kmalloc(16, GFP_KERNEL);
                if(map_banks[idx]->name == NULL) {
                        ret = -ENOMEM;
                        goto error_mem;
                }
                memset((void *)map_banks[idx]->name, 0, 16);

                sprintf(map_banks[idx]->name, "Amazon_se Bank %d", idx);
                map_banks[idx]->buswidth = 2;
                map_banks[idx]->read8 = amazon_se_read8;
                map_banks[idx]->read16 = amazon_se_read16;
                map_banks[idx]->read32 = amazon_se_read32;
                map_banks[idx]->copy_from = amazon_se_copy_from;
                map_banks[idx]->write8 = amazon_se_write8;
                map_banks[idx]->write16 = amazon_se_write16;
                map_banks[idx]->write32 = amazon_se_write32;
                map_banks[idx]->copy_to = amazon_se_copy_to;
                map_banks[idx]->map_priv_1 = start_scan_addr;
    /* make sure probing works */
                map_banks[idx]->size = 0x2000;
                printk("Amazon_se: probing address:%x\n",map_banks[idx]->map_priv_1);

                /* start to probe flash chips */
                probing = 1;
                mtd_banks[idx] = do_map_probe("cfi_probe", map_banks[idx]);
                probing = 0;
                if(mtd_banks[idx]) {
                        struct cfi_private *cfi;

                        mtd_banks[idx]->module = THIS_MODULE;
                        mtd_size += mtd_banks[idx]->size;
                        num_banks++;
                        cfi = (struct cfi_private *)map_banks[idx]->fldrv_priv;
                        /* Swap the address for correct access via EBU */
                        cfi->addr_unlock1 ^= 2;
                        cfi->addr_unlock2 ^= 2;
                        printk("%s: bank%ld, name:%s, size:%dbytes \n", __FUNCTION__, num_banks,
                        mtd_banks[idx]->name, mtd_banks[idx]->size);
                }
        }

        /* no supported flash chips found */
        if(!num_banks)
        {
                printk("Amazon_se: No support flash chips found!\n");
                ret = -ENXIO;
                goto error_mem;
        }

#ifdef CONFIG_MTD_PARTITIONS
        /*
         * Select Static partition definitions
         */
        part_banks[0].mtd_part = amazon_se_partitions;
        part_banks[0].type = "static image";
        part_banks[0].nums = NB_OF(amazon_se_partitions);
/*
        part_banks[1].mtd_part = amazon_se_fs_partitions;
        part_banks[1].type = "static file system";
        part_banks[1].nums = NB_OF(amazon_se_fs_partitions);
*/

        for(idx = 0; idx < num_banks ; idx++) {
#ifdef CONFIG_MTD_CMDLINE_PARTS
                sprintf(mtdid, "%d", idx);
                n = parse_cmdline_partitions(mtd_banks[idx],
                                             &part_banks[idx].mtd_part,
                                             mtdid);
                AMAZON_SE_MTD_DMSG ("%d command line partitions on bank %s\n",n, mtdid);
                if (n > 0) {
                        part_banks[idx].type = "command line";
                        part_banks[idx].nums = n;
                }
#endif
                if (part_banks[idx].nums == 0) {
                        printk (KERN_NOTICE "AMAZON_SE flash%d: "
                                "no partition info available, "
                                "registering whole flash at once\n", idx);
                        add_mtd_device(mtd_banks[idx]);
                } else {
                        printk (KERN_NOTICE "AMAZON_SE flash%d: "
                                "Using %s partition definition\n",
                                idx, part_banks[idx].type);
                        add_mtd_partitions(mtd_banks[idx],
                                           part_banks[idx].mtd_part,
                                           part_banks[idx].nums);
                }
        }
#else
        printk (KERN_NOTICE "AMAZON_SE flash: registering %d whole "
                "flash banks at once\n", num_banks);
        for(idx = 0 ; idx < num_banks ; idx++)
                add_mtd_device(mtd_banks[idx]);
#endif
        return 0;
error_mem:
        for(idx = 0 ; idx < FLASH_BANK_MAX ; idx++) {
                if(map_banks[idx] != NULL) {
                        if(map_banks[idx]->name != NULL) {
                                kfree(map_banks[idx]->name);
                                map_banks[idx]->name = NULL;
                        }
                        kfree(map_banks[idx]);
                        map_banks[idx] = NULL;
                }
        }
error:
        iounmap((void *)start_scan_addr);
        return ret;
}

static void __exit cleanup_amazon_se_mtd(void)
{
        unsigned int idx = 0;
        for(idx = 0 ; idx < num_banks ; idx++)
        {
                /* destroy mtd_info previously allocated */
                if (mtd_banks[idx]) {
                        del_mtd_partitions(mtd_banks[idx]);
                        map_destroy(mtd_banks[idx]);
                }
                /* release map_info not used anymore */
                kfree(map_banks[idx]->name);
                kfree(map_banks[idx]);
        }
        if (start_scan_addr) {
                iounmap((void *)start_scan_addr);
                start_scan_addr = 0;
        }
}

module_init(init_amazon_se_mtd);
module_exit(cleanup_amazon_se_mtd);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("MTD map driver for AMAZON_SE boards");
